@@ -34,12 +34,14 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
+import java.util.Random;
 import java.util.Set;
 import java.util.UUID;
 import java.util.concurrent.Callable;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.concurrent.ExecutorService;
+import java.util.concurrent.ThreadLocalRandom;
 import java.util.stream.Collectors;
 import java.util.stream.Stream;
 import org.apache.hadoop.conf.Configuration;
@@ -109,12 +111,9 @@ import org.apache.hadoop.hdfs.protocol.HdfsFileStatus;
 import org.apache.hadoop.hdfs.server.blockmanagement.BlockStoragePolicySuite;
 import org.apache.hadoop.io.NullWritable;
 import org.apache.hadoop.mapreduce.Job;
-import org.apache.hadoop.mapreduce.JobContext;
 import org.apache.hadoop.mapreduce.Mapper;
-import org.apache.hadoop.mapreduce.OutputCommitter;
 import org.apache.hadoop.mapreduce.RecordWriter;
 import org.apache.hadoop.mapreduce.TaskAttemptContext;
-import org.apache.hadoop.mapreduce.lib.output.FileOutputCommitter;
 import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
 import org.junit.ClassRule;
 import org.junit.Ignore;
@@ -540,19 +539,23 @@ public class TestHFileOutputFormat2 {
   }
 
   private byte[][] generateRandomStartKeys(int numKeys) {
+    Random random = ThreadLocalRandom.current();
     byte[][] ret = new byte[numKeys][];
     // first region start key is always empty
     ret[0] = HConstants.EMPTY_BYTE_ARRAY;
     for (int i = 1; i < numKeys; i++) {
-      ret[i] = PerformanceEvaluation.generateData(PerformanceEvaluation.DEFAULT_VALUE_LENGTH);
+      ret[i] =
+        PerformanceEvaluation.generateData(random, PerformanceEvaluation.DEFAULT_VALUE_LENGTH);
     }
     return ret;
   }
 
   private byte[][] generateRandomSplitKeys(int numKeys) {
+    Random random = ThreadLocalRandom.current();
     byte[][] ret = new byte[numKeys][];
     for (int i = 0; i < numKeys; i++) {
-      ret[i] = PerformanceEvaluation.generateData(PerformanceEvaluation.DEFAULT_VALUE_LENGTH);
+      ret[i] =
+        PerformanceEvaluation.generateData(random, PerformanceEvaluation.DEFAULT_VALUE_LENGTH);
     }
     return ret;
   }
@@ -596,22 +599,33 @@ public class TestHFileOutputFormat2 {
 
   private void doIncrementalLoadTest(boolean shouldChangeRegions, boolean shouldKeepLocality,
     boolean putSortReducer, String tableStr) throws Exception {
-    doIncrementalLoadTest(shouldChangeRegions, shouldKeepLocality, putSortReducer,
+    doIncrementalLoadTest(shouldChangeRegions, shouldKeepLocality, putSortReducer, false,
       Arrays.asList(tableStr));
   }
 
   @Test
   public void testMultiMRIncrementalLoadWithPutSortReducer() throws Exception {
     LOG.info("\nStarting test testMultiMRIncrementalLoadWithPutSortReducer\n");
-    doIncrementalLoadTest(false, false, true,
+    doIncrementalLoadTest(false, false, true, false,
+      Arrays.stream(TABLE_NAMES).map(TableName::getNameAsString).collect(Collectors.toList()));
+  }
+
+  @Test
+  public void testMultiMRIncrementalLoadWithPutSortReducerWithNamespaceInPath() throws Exception {
+    LOG.info("\nStarting test testMultiMRIncrementalLoadWithPutSortReducerWithNamespaceInPath\n");
+    doIncrementalLoadTest(false, false, true, true,
       Arrays.stream(TABLE_NAMES).map(TableName::getNameAsString).collect(Collectors.toList()));
   }
 
   private void doIncrementalLoadTest(boolean shouldChangeRegions, boolean shouldKeepLocality,
-    boolean putSortReducer, List<String> tableStr) throws Exception {
+    boolean putSortReducer, boolean shouldWriteToTableWithNamespace, List<String> tableStr)
+    throws Exception {
     util = new HBaseTestingUtility();
     Configuration conf = util.getConfiguration();
     conf.setBoolean(MultiTableHFileOutputFormat.LOCALITY_SENSITIVE_CONF_KEY, shouldKeepLocality);
+    if (shouldWriteToTableWithNamespace) {
+      conf.setBoolean(HFileOutputFormat2.TABLE_NAME_WITH_NAMESPACE_INCLUSIVE_KEY, true);
+    }
     int hostCount = 1;
     int regionNum = 5;
     if (shouldKeepLocality) {
@@ -648,15 +662,18 @@ public class TestHFileOutputFormat2 {
     Path testDir = util.getDataTestDirOnTestFS("testLocalMRIncrementalLoad");
     // Generate the bulk load files
     runIncrementalPELoad(conf, tableInfo, testDir, putSortReducer);
+    if (shouldWriteToTableWithNamespace) {
+      testDir = new Path(testDir, "default");
+    }
 
     for (Table tableSingle : allTables.values()) {
       // This doesn't write into the table, just makes files
       assertEquals("HFOF should not touch actual table", 0, util.countRows(tableSingle));
     }
     int numTableDirs = 0;
-    for (FileStatus tf : testDir.getFileSystem(conf).listStatus(testDir)) {
+    FileStatus[] fss = testDir.getFileSystem(conf).listStatus(testDir);
+    for (FileStatus tf : fss) {
       Path tablePath = testDir;
-
       if (writeMultipleTables) {
         if (allTables.containsKey(tf.getPath().getName())) {
           ++numTableDirs;
@@ -668,7 +685,8 @@ public class TestHFileOutputFormat2 {
 
       // Make sure that a directory was created for every CF
       int dir = 0;
-      for (FileStatus f : tablePath.getFileSystem(conf).listStatus(tablePath)) {
+      fss = tablePath.getFileSystem(conf).listStatus(tablePath);
+      for (FileStatus f : fss) {
         for (byte[] family : FAMILIES) {
           if (Bytes.toString(family).equals(f.getPath().getName())) {
             ++dir;
@@ -1580,59 +1598,6 @@ public class TestHFileOutputFormat2 {
     }
   }
 
-  @Test
-  public void itGetsWorkPathHadoop2() throws Exception {
-    Configuration conf = new Configuration(this.util.getConfiguration());
-    Job job = new Job(conf);
-    FileOutputCommitter committer =
-      new FileOutputCommitter(new Path("/test"), createTestTaskAttemptContext(job));
-    assertEquals(committer.getWorkPath(), HFileOutputFormat2.getWorkPath(committer));
-  }
-
-  @Test
-  public void itGetsWorkPathHadoo3() {
-    Hadoop3TestOutputCommitter committer = new Hadoop3TestOutputCommitter(new Path("/test"));
-    assertEquals(committer.getWorkPath(), HFileOutputFormat2.getWorkPath(committer));
-  }
-
-  static class Hadoop3TestOutputCommitter extends OutputCommitter {
-
-    Path path;
-
-    Hadoop3TestOutputCommitter(Path path) {
-      this.path = path;
-    }
-
-    public Path getWorkPath() {
-      return path;
-    }
-
-    @Override
-    public void setupJob(JobContext jobContext) throws IOException {
-
-    }
-
-    @Override
-    public void setupTask(TaskAttemptContext taskAttemptContext) throws IOException {
-
-    }
-
-    @Override
-    public boolean needsTaskCommit(TaskAttemptContext taskAttemptContext) throws IOException {
-      return false;
-    }
-
-    @Override
-    public void commitTask(TaskAttemptContext taskAttemptContext) throws IOException {
-
-    }
-
-    @Override
-    public void abortTask(TaskAttemptContext taskAttemptContext) throws IOException {
-
-    }
-  }
-
   private static class ConfigurationCaptorConnection implements Connection {
     private static final String UUID_KEY = "ConfigurationCaptorConnection.uuid";
 
@@ -1640,12 +1605,11 @@ public class TestHFileOutputFormat2 {
 
     private final Connection delegate;
 
-    public ConfigurationCaptorConnection(Configuration conf, ExecutorService es, User user)
-      throws IOException {
+    public ConfigurationCaptorConnection(Configuration conf, ExecutorService es, User user,
+      Map<String, byte[]> connectionAttributes) throws IOException {
       Configuration confForDelegate = new Configuration(conf);
       confForDelegate.unset(ClusterConnection.HBASE_CLIENT_CONNECTION_IMPL);
-      delegate = createConnection(confForDelegate, es, user);
-
+      delegate = createConnection(confForDelegate, es, user, connectionAttributes);
       final String uuid = conf.get(UUID_KEY);
       if (uuid != null) {
         confs.computeIfAbsent(UUID.fromString(uuid), u -> new CopyOnWriteArrayList<>()).add(conf);
