@@ -44,14 +44,12 @@ import org.apache.hadoop.hbase.client.Admin;
 import org.apache.hadoop.hbase.client.ColumnFamilyDescriptorBuilder;
 import org.apache.hadoop.hbase.client.Put;
 import org.apache.hadoop.hbase.client.RegionInfo;
-import org.apache.hadoop.hbase.client.SnapshotType;
 import org.apache.hadoop.hbase.client.Table;
 import org.apache.hadoop.hbase.client.TableDescriptorBuilder;
 import org.apache.hadoop.hbase.master.snapshot.SnapshotManager;
 import org.apache.hadoop.hbase.regionserver.StoreFileInfo;
 import org.apache.hadoop.hbase.testclassification.LargeTests;
 import org.apache.hadoop.hbase.testclassification.VerySlowMapReduceTests;
-import org.apache.hadoop.hbase.util.AbstractHBaseTool;
 import org.apache.hadoop.hbase.util.Bytes;
 import org.apache.hadoop.hbase.util.CommonFSUtils;
 import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
@@ -229,23 +227,6 @@ public class TestExportSnapshot {
     removeExportDir(copyDir);
   }
 
-  @Test
-  public void testExportWithChecksum() throws Exception {
-    // Test different schemes: input scheme is hdfs:// and output scheme is file://
-    // The checksum verification will fail
-    Path copyLocalDir = getLocalDestinationDir(TEST_UTIL);
-    testExportFileSystemState(TEST_UTIL.getConfiguration(), tableName, snapshotName, snapshotName,
-      tableNumFiles, TEST_UTIL.getDefaultRootDirPath(), copyLocalDir, false, false,
-      getBypassRegionPredicate(), false, true);
-
-    // Test same schemes: input scheme is hdfs:// and output scheme is hdfs://
-    // The checksum verification will success
-    Path copyHdfsDir = getHdfsDestinationDir();
-    testExportFileSystemState(TEST_UTIL.getConfiguration(), tableName, snapshotName, snapshotName,
-      tableNumFiles, TEST_UTIL.getDefaultRootDirPath(), copyHdfsDir, false, false,
-      getBypassRegionPredicate(), true, true);
-  }
-
   @Test
   public void testExportWithTargetName() throws Exception {
     final byte[] targetName = Bytes.toBytes("testExportWithTargetName");
@@ -284,39 +265,6 @@ public class TestExportSnapshot {
     }
   }
 
-  @Test
-  public void testExportExpiredSnapshot() throws Exception {
-    String name = "testExportExpiredSnapshot";
-    TableName tableName = TableName.valueOf(name);
-    String snapshotName = "snapshot-" + name;
-    createTable(tableName);
-    SnapshotTestingUtils.loadData(TEST_UTIL, tableName, 50, FAMILY);
-    Map<String, Object> properties = new HashMap<>();
-    properties.put("TTL", 10);
-    org.apache.hadoop.hbase.client.SnapshotDescription snapshotDescription =
-      new org.apache.hadoop.hbase.client.SnapshotDescription(snapshotName, tableName,
-        SnapshotType.FLUSH, null, EnvironmentEdgeManager.currentTime(), -1, properties);
-    admin.snapshot(snapshotDescription);
-    boolean isExist =
-      admin.listSnapshots().stream().anyMatch(ele -> snapshotName.equals(ele.getName()));
-    assertTrue(isExist);
-    int retry = 6;
-    while (
-      !SnapshotDescriptionUtils.isExpiredSnapshot(snapshotDescription.getTtl(),
-        snapshotDescription.getCreationTime(), EnvironmentEdgeManager.currentTime()) && retry > 0
-    ) {
-      retry--;
-      Thread.sleep(10 * 1000);
-    }
-    boolean isExpiredSnapshot =
-      SnapshotDescriptionUtils.isExpiredSnapshot(snapshotDescription.getTtl(),
-        snapshotDescription.getCreationTime(), EnvironmentEdgeManager.currentTime());
-    assertTrue(isExpiredSnapshot);
-    int res = runExportSnapshot(TEST_UTIL.getConfiguration(), snapshotName, snapshotName,
-      TEST_UTIL.getDefaultRootDirPath(), getHdfsDestinationDir(), false, false, false, true, true);
-    assertTrue(res == AbstractHBaseTool.EXIT_FAILURE);
-  }
-
   private void testExportFileSystemState(final TableName tableName, final byte[] snapshotName,
     final byte[] targetName, int filesExpected) throws Exception {
     testExportFileSystemState(tableName, snapshotName, targetName, filesExpected,
@@ -334,7 +282,7 @@ public class TestExportSnapshot {
     throws Exception {
     testExportFileSystemState(TEST_UTIL.getConfiguration(), tableName, snapshotName, targetName,
       filesExpected, TEST_UTIL.getDefaultRootDirPath(), copyDir, overwrite, resetTtl,
-      getBypassRegionPredicate(), true, false);
+      getBypassRegionPredicate(), true);
   }
 
   /**
@@ -343,15 +291,31 @@ public class TestExportSnapshot {
   protected static void testExportFileSystemState(final Configuration conf,
     final TableName tableName, final byte[] snapshotName, final byte[] targetName,
     final int filesExpected, final Path srcDir, Path rawTgtDir, final boolean overwrite,
-    final boolean resetTtl, final RegionPredicate bypassregionPredicate, final boolean success,
-    final boolean checksumVerify) throws Exception {
+    final boolean resetTtl, final RegionPredicate bypassregionPredicate, boolean success)
+    throws Exception {
     FileSystem tgtFs = rawTgtDir.getFileSystem(conf);
     FileSystem srcFs = srcDir.getFileSystem(conf);
     Path tgtDir = rawTgtDir.makeQualified(tgtFs.getUri(), tgtFs.getWorkingDirectory());
+    LOG.info("tgtFsUri={}, tgtDir={}, rawTgtDir={}, srcFsUri={}, srcDir={}", tgtFs.getUri(), tgtDir,
+      rawTgtDir, srcFs.getUri(), srcDir);
+    List<String> opts = new ArrayList<>();
+    opts.add("--snapshot");
+    opts.add(Bytes.toString(snapshotName));
+    opts.add("--copy-to");
+    opts.add(tgtDir.toString());
+    if (targetName != snapshotName) {
+      opts.add("--target");
+      opts.add(Bytes.toString(targetName));
+    }
+    if (overwrite) {
+      opts.add("--overwrite");
+    }
+    if (resetTtl) {
+      opts.add("--reset-ttl");
+    }
 
     // Export Snapshot
-    int res = runExportSnapshot(conf, Bytes.toString(snapshotName), Bytes.toString(targetName),
-      srcDir, rawTgtDir, overwrite, resetTtl, checksumVerify, true, true);
+    int res = run(conf, new ExportSnapshot(), opts.toArray(new String[opts.size()]));
     assertEquals("success " + success + ", res=" + res, success ? 0 : 1, res);
     if (!success) {
       final Path targetDir = new Path(HConstants.SNAPSHOT_DIR_NAME, Bytes.toString(targetName));
@@ -484,42 +448,4 @@ public class TestExportSnapshot {
     FileSystem fs = FileSystem.get(path.toUri(), new Configuration());
     fs.delete(path, true);
   }
-
-  private static int runExportSnapshot(final Configuration conf, final String sourceSnapshotName,
-    final String targetSnapshotName, final Path srcDir, Path rawTgtDir, final boolean overwrite,
-    final boolean resetTtl, final boolean checksumVerify, final boolean noSourceVerify,
-    final boolean noTargetVerify) throws Exception {
-    FileSystem tgtFs = rawTgtDir.getFileSystem(conf);
-    FileSystem srcFs = srcDir.getFileSystem(conf);
-    Path tgtDir = rawTgtDir.makeQualified(tgtFs.getUri(), tgtFs.getWorkingDirectory());
-    LOG.info("tgtFsUri={}, tgtDir={}, rawTgtDir={}, srcFsUri={}, srcDir={}", tgtFs.getUri(), tgtDir,
-      rawTgtDir, srcFs.getUri(), srcDir);
-    List<String> opts = new ArrayList<>();
-    opts.add("--snapshot");
-    opts.add(sourceSnapshotName);
-    opts.add("--copy-to");
-    opts.add(tgtDir.toString());
-    if (!targetSnapshotName.equals(sourceSnapshotName)) {
-      opts.add("--target");
-      opts.add(targetSnapshotName);
-    }
-    if (overwrite) {
-      opts.add("--overwrite");
-    }
-    if (resetTtl) {
-      opts.add("--reset-ttl");
-    }
-    if (!checksumVerify) {
-      opts.add("--no-checksum-verify");
-    }
-    if (!noSourceVerify) {
-      opts.add("--no-source-verify");
-    }
-    if (!noTargetVerify) {
-      opts.add("--no-target-verify");
-    }
-
-    // Export Snapshot
-    return run(conf, new ExportSnapshot(), opts.toArray(new String[opts.size()]));
-  }
 }
