@@ -112,7 +112,8 @@ public class RowResource extends ResourceBase {
           rowKey = CellUtil.cloneRow(value);
           rowModel = new RowModel(rowKey);
         }
-        rowModel.addCell(new CellModel(value));
+        rowModel.addCell(new CellModel(CellUtil.cloneFamily(value), CellUtil.cloneQualifier(value),
+          value.getTimestamp(), CellUtil.cloneValue(value)));
         if (++count > rowspec.getMaxValues()) {
           break;
         }
@@ -710,7 +711,8 @@ public class RowResource extends ResourceBase {
         CellSetModel rModel = new CellSetModel();
         RowModel rRowModel = new RowModel(result.getRow());
         for (Cell cell : result.listCells()) {
-          rRowModel.addCell(new CellModel(cell));
+          rRowModel.addCell(new CellModel(CellUtil.cloneFamily(cell), CellUtil.cloneQualifier(cell),
+            cell.getTimestamp(), CellUtil.cloneValue(cell)));
         }
         rModel.addRow(rRowModel);
         servlet.getMetrics().incrementSucessfulAppendRequests(1);
@@ -801,7 +803,8 @@ public class RowResource extends ResourceBase {
         CellSetModel rModel = new CellSetModel();
         RowModel rRowModel = new RowModel(result.getRow());
         for (Cell cell : result.listCells()) {
-          rRowModel.addCell(new CellModel(cell));
+          rRowModel.addCell(new CellModel(CellUtil.cloneFamily(cell), CellUtil.cloneQualifier(cell),
+            cell.getTimestamp(), CellUtil.cloneValue(cell)));
         }
         rModel.addRow(rowModel);
         servlet.getMetrics().incrementSucessfulIncrementRequests(1);
