@@ -17,8 +17,6 @@
  */
 package org.apache.hadoop.hbase.rest.model;
 
-import static org.apache.hadoop.hbase.rest.model.CellModel.MAGIC_LENGTH;
-
 import java.io.IOException;
 import java.io.Serializable;
 import java.util.ArrayList;
@@ -28,16 +26,13 @@ import javax.xml.bind.annotation.XmlAccessorType;
 import javax.xml.bind.annotation.XmlElement;
 import javax.xml.bind.annotation.XmlRootElement;
 import org.apache.hadoop.hbase.HConstants;
+import org.apache.hadoop.hbase.protobuf.ProtobufUtil;
 import org.apache.hadoop.hbase.rest.ProtobufMessageHandler;
-import org.apache.hadoop.hbase.rest.RestUtil;
 import org.apache.hadoop.hbase.rest.protobuf.generated.CellMessage.Cell;
 import org.apache.hadoop.hbase.rest.protobuf.generated.CellSetMessage.CellSet;
+import org.apache.hadoop.hbase.util.ByteStringer;
 import org.apache.yetus.audience.InterfaceAudience;
 
-import org.apache.hbase.thirdparty.com.google.protobuf.CodedInputStream;
-import org.apache.hbase.thirdparty.com.google.protobuf.Message;
-import org.apache.hbase.thirdparty.com.google.protobuf.UnsafeByteOperations;
-
 /**
  * Representation of a grouping of cells. May contain cells from more than one row. Encapsulates
  * RowModel and CellModel models.
@@ -72,7 +67,7 @@ import org.apache.hbase.thirdparty.com.google.protobuf.UnsafeByteOperations;
  * </pre>
  */
 @XmlRootElement(name = "CellSet")
-@XmlAccessorType(XmlAccessType.NONE)
+@XmlAccessorType(XmlAccessType.FIELD)
 @InterfaceAudience.Private
 public class CellSetModel implements Serializable, ProtobufMessageHandler {
   private static final long serialVersionUID = 1L;
@@ -109,25 +104,15 @@ public class CellSetModel implements Serializable, ProtobufMessageHandler {
   }
 
   @Override
-  public Message messageFromObject() {
+  public byte[] createProtobufOutput() {
     CellSet.Builder builder = CellSet.newBuilder();
     for (RowModel row : getRows()) {
       CellSet.Row.Builder rowBuilder = CellSet.Row.newBuilder();
-      if (row.getKeyLength() == MAGIC_LENGTH) {
-        rowBuilder.setKey(UnsafeByteOperations.unsafeWrap(row.getKey()));
-      } else {
-        rowBuilder.setKey(UnsafeByteOperations.unsafeWrap(row.getKeyArray(), row.getKeyOffset(),
-          row.getKeyLength()));
-      }
+      rowBuilder.setKey(ByteStringer.wrap(row.getKey()));
       for (CellModel cell : row.getCells()) {
         Cell.Builder cellBuilder = Cell.newBuilder();
-        cellBuilder.setColumn(UnsafeByteOperations.unsafeWrap(cell.getColumn()));
-        if (cell.getValueLength() == MAGIC_LENGTH) {
-          cellBuilder.setData(UnsafeByteOperations.unsafeWrap(cell.getValue()));
-        } else {
-          cellBuilder.setData(UnsafeByteOperations.unsafeWrap(cell.getValueArray(),
-            cell.getValueOffset(), cell.getValueLength()));
-        }
+        cellBuilder.setColumn(ByteStringer.wrap(cell.getColumn()));
+        cellBuilder.setData(ByteStringer.wrap(cell.getValue()));
         if (cell.hasUserTimestamp()) {
           cellBuilder.setTimestamp(cell.getTimestamp());
         }
@@ -135,13 +120,13 @@ public class CellSetModel implements Serializable, ProtobufMessageHandler {
       }
       builder.addRows(rowBuilder);
     }
-    return builder.build();
+    return builder.build().toByteArray();
   }
 
   @Override
-  public ProtobufMessageHandler getObjectFromMessage(CodedInputStream cis) throws IOException {
+  public ProtobufMessageHandler getObjectFromMessage(byte[] message) throws IOException {
     CellSet.Builder builder = CellSet.newBuilder();
-    RestUtil.mergeFrom(builder, cis);
+    ProtobufUtil.mergeFrom(builder, message);
     for (CellSet.Row row : builder.getRowsList()) {
       RowModel rowModel = new RowModel(row.getKey().toByteArray());
       for (Cell cell : row.getValuesList()) {
