@@ -21,6 +21,7 @@ import java.io.IOException;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hbase.ClusterMetrics;
 import org.apache.hadoop.hbase.HBaseInterfaceAudience;
 import org.apache.hadoop.hbase.MetaMutationAnnotation;
@@ -665,6 +666,46 @@ public interface MasterObserver {
     final TableName tableName, final byte[] splitRow) throws IOException {
   }
 
+  /**
+   * Called before the region is truncated.
+   * @param c          The environment to interact with the framework and master
+   * @param regionInfo The Region being truncated
+   */
+  @SuppressWarnings("unused")
+  default void preTruncateRegionAction(final ObserverContext<MasterCoprocessorEnvironment> c,
+    final RegionInfo regionInfo) {
+  }
+
+  /**
+   * Called before the truncate region procedure is called.
+   * @param c          The environment to interact with the framework and master
+   * @param regionInfo The Region being truncated
+   */
+  @SuppressWarnings("unused")
+  default void preTruncateRegion(final ObserverContext<MasterCoprocessorEnvironment> c,
+    RegionInfo regionInfo) {
+  }
+
+  /**
+   * Called after the truncate region procedure is called.
+   * @param c          The environment to interact with the framework and master
+   * @param regionInfo The Region being truncated
+   */
+  @SuppressWarnings("unused")
+  default void postTruncateRegion(final ObserverContext<MasterCoprocessorEnvironment> c,
+    RegionInfo regionInfo) {
+  }
+
+  /**
+   * Called post the region is truncated.
+   * @param c          The environment to interact with the framework and master
+   * @param regionInfo The Region To be truncated
+   */
+  @SuppressWarnings("unused")
+  default void postTruncateRegionAction(final ObserverContext<MasterCoprocessorEnvironment> c,
+    final RegionInfo regionInfo) {
+  }
+
   /**
    * Called after the region is split.
    * @param c           the environment to interact with the framework and master
@@ -1400,76 +1441,6 @@ public interface MasterObserver {
     Set<Address> servers) throws IOException {
   }
 
-  /**
-   * Called before getting region server group info of the passed groupName.
-   * @param ctx       the environment to interact with the framework and master
-   * @param groupName name of the group to get RSGroupInfo for
-   */
-  default void preGetRSGroupInfo(final ObserverContext<MasterCoprocessorEnvironment> ctx,
-    final String groupName) throws IOException {
-  }
-
-  /**
-   * Called after getting region server group info of the passed groupName.
-   * @param ctx       the environment to interact with the framework and master
-   * @param groupName name of the group to get RSGroupInfo for
-   */
-  default void postGetRSGroupInfo(final ObserverContext<MasterCoprocessorEnvironment> ctx,
-    final String groupName) throws IOException {
-  }
-
-  /**
-   * Called before getting region server group info of the passed tableName.
-   * @param ctx       the environment to interact with the framework and master
-   * @param tableName name of the table to get RSGroupInfo for
-   */
-  default void preGetRSGroupInfoOfTable(final ObserverContext<MasterCoprocessorEnvironment> ctx,
-    final TableName tableName) throws IOException {
-  }
-
-  /**
-   * Called after getting region server group info of the passed tableName.
-   * @param ctx       the environment to interact with the framework and master
-   * @param tableName name of the table to get RSGroupInfo for
-   */
-  default void postGetRSGroupInfoOfTable(final ObserverContext<MasterCoprocessorEnvironment> ctx,
-    final TableName tableName) throws IOException {
-  }
-
-  /**
-   * Called before listing region server group information.
-   * @param ctx the environment to interact with the framework and master
-   */
-  default void preListRSGroups(final ObserverContext<MasterCoprocessorEnvironment> ctx)
-    throws IOException {
-  }
-
-  /**
-   * Called after listing region server group information.
-   * @param ctx the environment to interact with the framework and master
-   */
-  default void postListRSGroups(final ObserverContext<MasterCoprocessorEnvironment> ctx)
-    throws IOException {
-  }
-
-  /**
-   * Called before getting region server group info of the passed server.
-   * @param ctx    the environment to interact with the framework and master
-   * @param server server to get RSGroupInfo for
-   */
-  default void preGetRSGroupInfoOfServer(final ObserverContext<MasterCoprocessorEnvironment> ctx,
-    final Address server) throws IOException {
-  }
-
-  /**
-   * Called after getting region server group info of the passed server.
-   * @param ctx    the environment to interact with the framework and master
-   * @param server server to get RSGroupInfo for
-   */
-  default void postGetRSGroupInfoOfServer(final ObserverContext<MasterCoprocessorEnvironment> ctx,
-    final Address server) throws IOException {
-  }
-
   /**
    * Called before add a replication peer
    * @param ctx        the environment to interact with the framework and master
@@ -1878,4 +1849,24 @@ public interface MasterObserver {
   default void postUpdateRSGroupConfig(final ObserverContext<MasterCoprocessorEnvironment> ctx,
     final String groupName, final Map<String, String> configuration) throws IOException {
   }
+
+  /*
+   * Called before reloading the HMaster's {@link Configuration} from disk
+   * @param ctx the coprocessor instance's environment
+   * @param preReloadConf the {@link Configuration} in use prior to reload
+   * @throws IOException if you need to signal an IO error
+   */
+  default void preUpdateMasterConfiguration(ObserverContext<MasterCoprocessorEnvironment> ctx,
+    Configuration preReloadConf) throws IOException {
+  }
+
+  /**
+   * Called after reloading the HMaster's {@link Configuration} from disk
+   * @param ctx            the coprocessor instance's environment
+   * @param postReloadConf the {@link Configuration} that was loaded
+   * @throws IOException if you need to signal an IO error
+   */
+  default void postUpdateMasterConfiguration(ObserverContext<MasterCoprocessorEnvironment> ctx,
+    Configuration postReloadConf) throws IOException {
+  }
 }
