@@ -60,7 +60,7 @@ public class ExecutorService {
   private static final Logger LOG = LoggerFactory.getLogger(ExecutorService.class);
 
   // hold the all the executors created in a map addressable by their names
-  private final ConcurrentHashMap<String, Executor> executorMap = new ConcurrentHashMap<>();
+  private final ConcurrentMap<String, Executor> executorMap = new ConcurrentHashMap<>();
 
   // Name of the server hosting this executor service.
   private final String servername;
@@ -84,18 +84,16 @@ public class ExecutorService {
    */
   public void startExecutorService(final ExecutorConfig config) {
     final String name = config.getName();
-    if (this.executorMap.get(name) != null) {
-      throw new RuntimeException(
-        "An executor service with the name " + name + " is already running!");
-    }
-    Executor hbes = new Executor(config);
-    if (this.executorMap.putIfAbsent(name, hbes) != null) {
-      throw new RuntimeException(
-        "An executor service with the name " + name + " is already running (2)!");
-    }
-    LOG.debug("Starting executor service name=" + name + ", corePoolSize="
-      + hbes.threadPoolExecutor.getCorePoolSize() + ", maxPoolSize="
-      + hbes.threadPoolExecutor.getMaximumPoolSize());
+    Executor hbes = this.executorMap.compute(name, (key, value) -> {
+      if (value != null) {
+        throw new RuntimeException(
+          "An executor service with the name " + key + " is already running!");
+      }
+      return new Executor(config);
+    });
+
+    LOG.debug("Starting executor service name={}, corePoolSize={}, maxPoolSize={}", name,
+      hbes.threadPoolExecutor.getCorePoolSize(), hbes.threadPoolExecutor.getMaximumPoolSize());
   }
 
   boolean isExecutorServiceRunning(String name) {
@@ -259,7 +257,7 @@ public class ExecutorService {
     }
 
     /**
-     * Submit the event to the queue for handling. n
+     * Submit the event to the queue for handling.
      */
     void submit(final EventHandler event) {
       // If there is a listener for this type, make sure we call the before
