@@ -17,18 +17,22 @@
  */
 package org.apache.hadoop.hbase.io.hfile.bucket;
 
+import static org.apache.hadoop.hbase.io.hfile.CacheConfig.BUCKETCACHE_PERSIST_INTERVAL_KEY;
+
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.Comparator;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.NavigableSet;
+import java.util.Optional;
 import java.util.PriorityQueue;
 import java.util.Set;
 import java.util.concurrent.ArrayBlockingQueue;
@@ -48,8 +52,11 @@ import java.util.concurrent.locks.ReentrantReadWriteLock;
 import java.util.function.Consumer;
 import java.util.function.Function;
 import org.apache.commons.io.IOUtils;
+import org.apache.commons.lang3.mutable.MutableInt;
 import org.apache.hadoop.conf.Configuration;
+import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.hbase.HBaseConfiguration;
+import org.apache.hadoop.hbase.HBaseIOException;
 import org.apache.hadoop.hbase.TableName;
 import org.apache.hadoop.hbase.client.Admin;
 import org.apache.hadoop.hbase.io.ByteBuffAllocator;
@@ -60,9 +67,11 @@ import org.apache.hadoop.hbase.io.hfile.BlockCacheKey;
 import org.apache.hadoop.hbase.io.hfile.BlockCacheUtil;
 import org.apache.hadoop.hbase.io.hfile.BlockPriority;
 import org.apache.hadoop.hbase.io.hfile.BlockType;
+import org.apache.hadoop.hbase.io.hfile.CacheConfig;
 import org.apache.hadoop.hbase.io.hfile.CacheStats;
 import org.apache.hadoop.hbase.io.hfile.Cacheable;
 import org.apache.hadoop.hbase.io.hfile.CachedBlock;
+import org.apache.hadoop.hbase.io.hfile.CombinedBlockCache;
 import org.apache.hadoop.hbase.io.hfile.HFileBlock;
 import org.apache.hadoop.hbase.io.hfile.HFileContext;
 import org.apache.hadoop.hbase.nio.ByteBuff;
@@ -72,6 +81,7 @@ import org.apache.hadoop.hbase.util.Bytes;
 import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.apache.hadoop.hbase.util.IdReadWriteLock;
 import org.apache.hadoop.hbase.util.IdReadWriteLock.ReferenceType;
+import org.apache.hadoop.hbase.util.Pair;
 import org.apache.hadoop.util.StringUtils;
 import org.apache.yetus.audience.InterfaceAudience;
 import org.slf4j.Logger;
@@ -134,8 +144,25 @@ public class BucketCache implements BlockCache, HeapSize {
 
   // Store the block in this map before writing it to cache
   transient final RAMCache ramCache;
+
   // In this map, store the block's meta data like offset, length
-  transient ConcurrentHashMap<BlockCacheKey, BucketEntry> backingMap;
+  transient Map<BlockCacheKey, BucketEntry> backingMap;
+
+  private AtomicBoolean backingMapValidated = new AtomicBoolean(false);
+
+  /**
+   * Map of hFile -> Region -> File size. This map is used to track all files completed prefetch,
+   * together with the region those belong to and the total cached size for the
+   * region.TestBlockEvictionOnRegionMovement
+   */
+  final Map<String, Pair<String, Long>> fullyCachedFiles = new ConcurrentHashMap<>();
+  /**
+   * Map of region -> total size of the region prefetched on this region server. This is the total
+   * size of hFiles for this region prefetched on this region server
+   */
+  final Map<String, Long> regionCachedSizeMap = new ConcurrentHashMap<>();
+
+  private BucketCachePersister cachePersister;
 
   /**
    * Flag if the cache is enabled or not... We shut it off if there are IO errors for some time, so
@@ -168,8 +195,8 @@ public class BucketCache implements BlockCache, HeapSize {
   private static final int DEFAULT_CACHE_WAIT_TIME = 50;
 
   private final BucketCacheStats cacheStats = new BucketCacheStats();
-
   private final String persistencePath;
+  static AtomicBoolean isCacheInconsistent = new AtomicBoolean(false);
   private final long cacheCapacity;
   /** Approximate block size */
   private final long blockSize;
@@ -191,13 +218,8 @@ public class BucketCache implements BlockCache, HeapSize {
    */
   transient final IdReadWriteLock<Long> offsetLock = new IdReadWriteLock<>(ReferenceType.SOFT);
 
-  final NavigableSet<BlockCacheKey> blocksByHFile = new ConcurrentSkipListSet<>((a, b) -> {
-    int nameComparison = a.getHfileName().compareTo(b.getHfileName());
-    if (nameComparison != 0) {
-      return nameComparison;
-    }
-    return Long.compare(a.getOffset(), b.getOffset());
-  });
+  NavigableSet<BlockCacheKey> blocksByHFile = new ConcurrentSkipListSet<>(
+    Comparator.comparing(BlockCacheKey::getHfileName).thenComparingLong(BlockCacheKey::getOffset));
 
   /** Statistics thread schedule pool (for heavy debugging, could remove) */
   private transient final ScheduledExecutorService scheduleThreadPool =
@@ -228,6 +250,8 @@ public class BucketCache implements BlockCache, HeapSize {
   /** In-memory bucket size */
   private float memoryFactor;
 
+  private long bucketcachePersistInterval;
+
   private static final String FILE_VERIFY_ALGORITHM =
     "hbase.bucketcache.persistent.file.integrity.check.algorithm";
   private static final String DEFAULT_FILE_VERIFY_ALGORITHM = "MD5";
@@ -274,6 +298,7 @@ public class BucketCache implements BlockCache, HeapSize {
     this.memoryFactor = conf.getFloat(MEMORY_FACTOR_CONFIG_NAME, DEFAULT_MEMORY_FACTOR);
     this.queueAdditionWaitTime =
       conf.getLong(QUEUE_ADDITION_WAIT_TIME, DEFAULT_QUEUE_ADDITION_WAIT_TIME);
+    this.bucketcachePersistInterval = conf.getLong(BUCKETCACHE_PERSIST_INTERVAL_KEY, 1000);
 
     sanityCheckConfigs();
 
@@ -288,7 +313,6 @@ public class BucketCache implements BlockCache, HeapSize {
 
     this.allocFailLogPrevTs = 0;
 
-    bucketAllocator = new BucketAllocator(capacity, bucketSizes);
     for (int i = 0; i < writerThreads.length; ++i) {
       writerQueues.add(new ArrayBlockingQueue<>(writerQLen));
     }
@@ -298,12 +322,22 @@ public class BucketCache implements BlockCache, HeapSize {
 
     this.backingMap = new ConcurrentHashMap<>((int) blockNumCapacity);
 
-    if (ioEngine.isPersistent() && persistencePath != null) {
+    if (isCachePersistent()) {
+      if (ioEngine instanceof FileIOEngine) {
+        startBucketCachePersisterThread();
+      }
       try {
         retrieveFromFile(bucketSizes);
       } catch (IOException ioex) {
-        LOG.error("Can't restore from file[" + persistencePath + "] because of ", ioex);
+        LOG.error("Can't restore from file[{}] because of ", persistencePath, ioex);
+        backingMap.clear();
+        fullyCachedFiles.clear();
+        backingMapValidated.set(true);
+        bucketAllocator = new BucketAllocator(capacity, bucketSizes);
+        regionCachedSizeMap.clear();
       }
+    } else {
+      bucketAllocator = new BucketAllocator(capacity, bucketSizes);
     }
     final String threadName = Thread.currentThread().getName();
     this.cacheEnabled = true;
@@ -355,6 +389,13 @@ public class BucketCache implements BlockCache, HeapSize {
     }
   }
 
+  void startBucketCachePersisterThread() {
+    LOG.info("Starting BucketCachePersisterThread");
+    cachePersister = new BucketCachePersister(this, bucketcachePersistInterval);
+    cachePersister.setDaemon(true);
+    cachePersister.start();
+  }
+
   boolean isCacheEnabled() {
     return this.cacheEnabled;
   }
@@ -399,6 +440,10 @@ public class BucketCache implements BlockCache, HeapSize {
     }
   }
 
+  public boolean isCachePersistenceEnabled() {
+    return persistencePath != null;
+  }
+
   /**
    * Cache the block with the specified name and buffer.
    * @param cacheKey block's cache key
@@ -466,10 +511,13 @@ public class BucketCache implements BlockCache, HeapSize {
     if (!cacheEnabled) {
       return;
     }
+    if (cacheKey.getBlockType() == null && cachedItem.getBlockType() != null) {
+      cacheKey.setBlockType(cachedItem.getBlockType());
+    }
     LOG.trace("Caching key={}, item={}", cacheKey, cachedItem);
     // Stuff the entry into the RAM cache so it can get drained to the persistent store
-    RAMQueueEntry re =
-      new RAMQueueEntry(cacheKey, cachedItem, accessCount.incrementAndGet(), inMemory);
+    RAMQueueEntry re = new RAMQueueEntry(cacheKey, cachedItem, accessCount.incrementAndGet(),
+      inMemory, isCachePersistent() && ioEngine instanceof FileIOEngine);
     /**
      * Don't use ramCache.put(cacheKey, re) here. because there may be a existing entry with same
      * key in ramCache, the heap size of bucket cache need to update if replacing entry from
@@ -553,6 +601,13 @@ public class BucketCache implements BlockCache, HeapSize {
           }
           return cachedBlock;
         }
+      } catch (HBaseIOException hioex) {
+        // When using file io engine persistent cache,
+        // the cache map state might differ from the actual cache. If we reach this block,
+        // we should remove the cache key entry from the backing map
+        backingMap.remove(key);
+        removeFileFromPrefetch(key.getHfileName());
+        LOG.debug("Failed to fetch block for cache key: {}.", key, hioex);
       } catch (IOException ioex) {
         LOG.error("Failed reading block " + key + " from bucket cache", ioex);
         checkIOErrorIsTolerated();
@@ -575,10 +630,16 @@ public class BucketCache implements BlockCache, HeapSize {
     blocksByHFile.remove(cacheKey);
     if (decrementBlockNumber) {
       this.blockNumber.decrement();
+      if (ioEngine.isPersistent()) {
+        removeFileFromPrefetch(cacheKey.getHfileName());
+      }
     }
     if (evictedByEvictionProcess) {
       cacheStats.evicted(bucketEntry.getCachedTime(), cacheKey.isPrimary());
     }
+    if (ioEngine.isPersistent()) {
+      setCacheInconsistent(true);
+    }
   }
 
   /**
@@ -639,6 +700,8 @@ public class BucketCache implements BlockCache, HeapSize {
     } else {
       return bucketEntryToUse.withWriteLock(offsetLock, () -> {
         if (backingMap.remove(cacheKey, bucketEntryToUse)) {
+          LOG.debug("removed key {} from back map with offset lock {} in the evict process",
+            cacheKey, bucketEntryToUse.offset());
           blockEvicted(cacheKey, bucketEntryToUse, !existedInRamCache, evictedByEvictionProcess);
           return true;
         }
@@ -703,6 +766,14 @@ public class BucketCache implements BlockCache, HeapSize {
     });
   }
 
+  public boolean isCacheInconsistent() {
+    return isCacheInconsistent.get();
+  }
+
+  public void setCacheInconsistent(boolean setCacheInconsistent) {
+    isCacheInconsistent.set(setCacheInconsistent);
+  }
+
   /*
    * Statistics thread. Periodically output cache statistics to the log.
    */
@@ -1146,6 +1217,9 @@ public class BucketCache implements BlockCache, HeapSize {
       // Only add if non-null entry.
       if (bucketEntries[i] != null) {
         putIntoBackingMap(key, bucketEntries[i]);
+        if (ioEngine.isPersistent()) {
+          setCacheInconsistent(true);
+        }
       }
       // Always remove from ramCache even if we failed adding it to the block cache above.
       boolean existed = ramCache.remove(key, re -> {
@@ -1195,28 +1269,53 @@ public class BucketCache implements BlockCache, HeapSize {
    */
   @edu.umd.cs.findbugs.annotations.SuppressWarnings(value = "OBL_UNSATISFIED_OBLIGATION",
       justification = "false positive, try-with-resources ensures close is called.")
-  private void persistToFile() throws IOException {
-    assert !cacheEnabled;
-    if (!ioEngine.isPersistent()) {
+  void persistToFile() throws IOException {
+    LOG.debug("Thread {} started persisting bucket cache to file",
+      Thread.currentThread().getName());
+    if (!isCachePersistent()) {
       throw new IOException("Attempt to persist non-persistent cache mappings!");
     }
-    try (FileOutputStream fos = new FileOutputStream(persistencePath, false)) {
+    File tempPersistencePath = new File(persistencePath + EnvironmentEdgeManager.currentTime());
+    try (FileOutputStream fos = new FileOutputStream(tempPersistencePath, false)) {
       fos.write(ProtobufMagic.PB_MAGIC);
       BucketProtoUtils.toPB(this).writeDelimitedTo(fos);
+    } catch (IOException e) {
+      LOG.error("Failed to persist bucket cache to file", e);
+      throw e;
     }
+    LOG.debug("Thread {} finished persisting bucket cache to file, renaming",
+      Thread.currentThread().getName());
+    if (!tempPersistencePath.renameTo(new File(persistencePath))) {
+      LOG.warn("Failed to commit cache persistent file. We might lose cached blocks if "
+        + "RS crashes/restarts before we successfully checkpoint again.");
+    }
+  }
+
+  public boolean isCachePersistent() {
+    return ioEngine.isPersistent() && persistencePath != null;
+  }
+
+  public Map<String, Long> getRegionCachedInfo() {
+    return Collections.unmodifiableMap(regionCachedSizeMap);
   }
 
   /**
    * @see #persistToFile()
    */
   private void retrieveFromFile(int[] bucketSizes) throws IOException {
+    LOG.info("Started retrieving bucket cache from file");
     File persistenceFile = new File(persistencePath);
     if (!persistenceFile.exists()) {
+      LOG.warn("Persistence file missing! "
+        + "It's ok if it's first run after enabling persistent cache.");
+      bucketAllocator = new BucketAllocator(cacheCapacity, bucketSizes, backingMap, realCacheSize);
+      blockNumber.add(backingMap.size());
+      backingMapValidated.set(true);
       return;
     }
     assert !cacheEnabled;
 
-    try (FileInputStream in = deleteFileOnClose(persistenceFile)) {
+    try (FileInputStream in = new FileInputStream(persistenceFile)) {
       int pblen = ProtobufMagic.lengthOfPBMagic();
       byte[] pbuf = new byte[pblen];
       IOUtils.readFully(in, pbuf, 0, pblen);
@@ -1229,6 +1328,30 @@ public class BucketCache implements BlockCache, HeapSize {
       parsePB(BucketCacheProtos.BucketCacheEntry.parseDelimitedFrom(in));
       bucketAllocator = new BucketAllocator(cacheCapacity, bucketSizes, backingMap, realCacheSize);
       blockNumber.add(backingMap.size());
+      LOG.info("Bucket cache retrieved from file successfully");
+    }
+  }
+
+  private void updateRegionSizeMapWhileRetrievingFromFile() {
+    // Update the regionCachedSizeMap with the region size while restarting the region server
+    if (LOG.isDebugEnabled()) {
+      LOG.debug("Updating region size map after retrieving cached file list");
+      dumpPrefetchList();
+    }
+    regionCachedSizeMap.clear();
+    fullyCachedFiles.forEach((hFileName, hFileSize) -> {
+      // Get the region name for each file
+      String regionEncodedName = hFileSize.getFirst();
+      long cachedFileSize = hFileSize.getSecond();
+      regionCachedSizeMap.merge(regionEncodedName, cachedFileSize,
+        (oldpf, fileSize) -> oldpf + fileSize);
+    });
+  }
+
+  private void dumpPrefetchList() {
+    for (Map.Entry<String, Pair<String, Long>> outerEntry : fullyCachedFiles.entrySet()) {
+      LOG.debug("Cached File Entry:<{},<{},{}>>", outerEntry.getKey(),
+        outerEntry.getValue().getFirst(), outerEntry.getValue().getSecond());
     }
   }
 
@@ -1293,16 +1416,57 @@ public class BucketCache implements BlockCache, HeapSize {
   }
 
   private void parsePB(BucketCacheProtos.BucketCacheEntry proto) throws IOException {
+    Pair<ConcurrentHashMap<BlockCacheKey, BucketEntry>, NavigableSet<BlockCacheKey>> pair =
+      BucketProtoUtils.fromPB(proto.getDeserializersMap(), proto.getBackingMap(),
+        this::createRecycler);
+    backingMap = pair.getFirst();
+    blocksByHFile = pair.getSecond();
+    fullyCachedFiles.clear();
+    fullyCachedFiles.putAll(BucketProtoUtils.fromPB(proto.getCachedFilesMap()));
     if (proto.hasChecksum()) {
-      ((PersistentIOEngine) ioEngine).verifyFileIntegrity(proto.getChecksum().toByteArray(),
-        algorithm);
+      try {
+        ((PersistentIOEngine) ioEngine).verifyFileIntegrity(proto.getChecksum().toByteArray(),
+          algorithm);
+        backingMapValidated.set(true);
+      } catch (IOException e) {
+        LOG.warn("Checksum for cache file failed. "
+          + "We need to validate each cache key in the backing map. "
+          + "This may take some time, so we'll do it in a background thread,");
+        Runnable cacheValidator = () -> {
+          while (bucketAllocator == null) {
+            try {
+              Thread.sleep(50);
+            } catch (InterruptedException ex) {
+              throw new RuntimeException(ex);
+            }
+          }
+          long startTime = EnvironmentEdgeManager.currentTime();
+          int totalKeysOriginally = backingMap.size();
+          for (Map.Entry<BlockCacheKey, BucketEntry> keyEntry : backingMap.entrySet()) {
+            try {
+              ((FileIOEngine) ioEngine).checkCacheTime(keyEntry.getValue());
+            } catch (IOException e1) {
+              LOG.debug("Check for key {} failed. Evicting.", keyEntry.getKey());
+              evictBlock(keyEntry.getKey());
+              removeFileFromPrefetch(keyEntry.getKey().getHfileName());
+            }
+          }
+          backingMapValidated.set(true);
+          LOG.info("Finished validating {} keys in the backing map. Recovered: {}. This took {}ms.",
+            totalKeysOriginally, backingMap.size(),
+            (EnvironmentEdgeManager.currentTime() - startTime));
+        };
+        Thread t = new Thread(cacheValidator);
+        t.setDaemon(true);
+        t.start();
+      }
     } else {
       // if has not checksum, it means the persistence file is old format
       LOG.info("Persistent file is old format, it does not support verifying file integrity!");
+      backingMapValidated.set(true);
     }
+    updateRegionSizeMapWhileRetrievingFromFile();
     verifyCapacityAndClasses(proto.getCacheCapacity(), proto.getIoClass(), proto.getMapClass());
-    backingMap = BucketProtoUtils.fromPB(proto.getDeserializersMap(), proto.getBackingMap(),
-      this::createRecycler);
   }
 
   /**
@@ -1329,6 +1493,7 @@ public class BucketCache implements BlockCache, HeapSize {
    */
   private void disableCache() {
     if (!cacheEnabled) return;
+    LOG.info("Disabling cache");
     cacheEnabled = false;
     ioEngine.shutdown();
     this.scheduleThreadPool.shutdown();
@@ -1339,6 +1504,8 @@ public class BucketCache implements BlockCache, HeapSize {
       // If persistent ioengine and a path, we will serialize out the backingMap.
       this.backingMap.clear();
       this.blocksByHFile.clear();
+      this.fullyCachedFiles.clear();
+      this.regionCachedSizeMap.clear();
     }
   }
 
@@ -1355,6 +1522,13 @@ public class BucketCache implements BlockCache, HeapSize {
     if (ioEngine.isPersistent() && persistencePath != null) {
       try {
         join();
+        if (cachePersister != null) {
+          LOG.info("Shutting down cache persister thread.");
+          cachePersister.shutdown();
+          while (cachePersister.isAlive()) {
+            Thread.sleep(10);
+          }
+        }
         persistToFile();
       } catch (IOException ex) {
         LOG.error("Unable to persist data on exit: " + ex.toString(), ex);
@@ -1364,6 +1538,17 @@ public class BucketCache implements BlockCache, HeapSize {
     }
   }
 
+  /**
+   * Needed mostly for UTs that might run in the same VM and create different BucketCache instances
+   * on different UT methods.
+   */
+  @Override
+  protected void finalize() {
+    if (cachePersister != null && !cachePersister.isInterrupted()) {
+      cachePersister.interrupt();
+    }
+  }
+
   @Override
   public CacheStats getStats() {
     return cacheStats;
@@ -1420,19 +1605,22 @@ public class BucketCache implements BlockCache, HeapSize {
    */
   @Override
   public int evictBlocksByHfileName(String hfileName) {
-    Set<BlockCacheKey> keySet = blocksByHFile.subSet(new BlockCacheKey(hfileName, Long.MIN_VALUE),
-      true, new BlockCacheKey(hfileName, Long.MAX_VALUE), true);
-
+    removeFileFromPrefetch(hfileName);
+    Set<BlockCacheKey> keySet = getAllCacheKeysForFile(hfileName);
     int numEvicted = 0;
     for (BlockCacheKey key : keySet) {
       if (evictBlock(key)) {
         ++numEvicted;
       }
     }
-
     return numEvicted;
   }
 
+  private Set<BlockCacheKey> getAllCacheKeysForFile(String hfileName) {
+    return blocksByHFile.subSet(new BlockCacheKey(hfileName, Long.MIN_VALUE), true,
+      new BlockCacheKey(hfileName, Long.MAX_VALUE), true);
+  }
+
   /**
    * Used to group bucket entries into priority buckets. There will be a BucketEntryGroup for each
    * priority (single, multi, memory). Once bucketed, the eviction algorithm takes the appropriate
@@ -1490,12 +1678,15 @@ public class BucketCache implements BlockCache, HeapSize {
     private final Cacheable data;
     private long accessCounter;
     private boolean inMemory;
+    private boolean isCachePersistent;
 
-    RAMQueueEntry(BlockCacheKey bck, Cacheable data, long accessCounter, boolean inMemory) {
+    RAMQueueEntry(BlockCacheKey bck, Cacheable data, long accessCounter, boolean inMemory,
+      boolean isCachePersistent) {
       this.key = bck;
       this.data = data;
       this.accessCounter = accessCounter;
       this.inMemory = inMemory;
+      this.isCachePersistent = isCachePersistent;
     }
 
     public Cacheable getData() {
@@ -1525,19 +1716,35 @@ public class BucketCache implements BlockCache, HeapSize {
       if (len == 0) {
         return null;
       }
+      if (isCachePersistent && data instanceof HFileBlock) {
+        len += Long.BYTES; // we need to record the cache time for consistency check in case of
+                           // recovery
+      }
       long offset = alloc.allocateBlock(len);
       boolean succ = false;
       BucketEntry bucketEntry = null;
       try {
-        bucketEntry = new BucketEntry(offset, len, accessCounter, inMemory, createRecycler,
-          getByteBuffAllocator());
+        int diskSizeWithHeader = (data instanceof HFileBlock)
+          ? ((HFileBlock) data).getOnDiskSizeWithHeader()
+          : data.getSerializedLength();
+        bucketEntry = new BucketEntry(offset, len, diskSizeWithHeader, accessCounter, inMemory,
+          createRecycler, getByteBuffAllocator());
         bucketEntry.setDeserializerReference(data.getDeserializer());
         if (data instanceof HFileBlock) {
           // If an instance of HFileBlock, save on some allocations.
           HFileBlock block = (HFileBlock) data;
           ByteBuff sliceBuf = block.getBufferReadOnly();
           block.getMetaData(metaBuff);
-          ioEngine.write(sliceBuf, offset);
+          // adds the cache time prior to the block and metadata part
+          if (isCachePersistent) {
+            ByteBuffer buffer = ByteBuffer.allocate(Long.BYTES);
+            buffer.putLong(bucketEntry.getCachedTime());
+            buffer.rewind();
+            ioEngine.write(buffer, offset);
+            ioEngine.write(sliceBuf, (offset + Long.BYTES));
+          } else {
+            ioEngine.write(sliceBuf, offset);
+          }
           ioEngine.write(metaBuff, offset + len - metaBuff.limit());
         } else {
           // Only used for testing.
@@ -1694,6 +1901,10 @@ public class BucketCache implements BlockCache, HeapSize {
     return memoryFactor;
   }
 
+  public String getPersistencePath() {
+    return persistencePath;
+  }
+
   /**
    * Wrapped the delegate ConcurrentMap with maintaining its block's reference count.
    */
@@ -1770,5 +1981,175 @@ public class BucketCache implements BlockCache, HeapSize {
         re.getData().release();
       }
     }
+
+    public boolean hasBlocksForFile(String fileName) {
+      return delegate.keySet().stream().filter(key -> key.getHfileName().equals(fileName))
+        .findFirst().isPresent();
+    }
   }
+
+  public Map<BlockCacheKey, BucketEntry> getBackingMap() {
+    return backingMap;
+  }
+
+  public AtomicBoolean getBackingMapValidated() {
+    return backingMapValidated;
+  }
+
+  @Override
+  public Optional<Map<String, Pair<String, Long>>> getFullyCachedFiles() {
+    return Optional.of(fullyCachedFiles);
+  }
+
+  public static Optional<BucketCache> getBucketCacheFromCacheConfig(CacheConfig cacheConf) {
+    if (cacheConf.getBlockCache().isPresent()) {
+      BlockCache bc = cacheConf.getBlockCache().get();
+      if (bc instanceof CombinedBlockCache) {
+        BlockCache l2 = ((CombinedBlockCache) bc).getSecondLevelCache();
+        if (l2 instanceof BucketCache) {
+          return Optional.of((BucketCache) l2);
+        }
+      } else if (bc instanceof BucketCache) {
+        return Optional.of((BucketCache) bc);
+      }
+    }
+    return Optional.empty();
+  }
+
+  @Override
+  public void notifyFileCachingCompleted(Path fileName, int totalBlockCount, int dataBlockCount,
+    long size) {
+    // block eviction may be happening in the background as prefetch runs,
+    // so we need to count all blocks for this file in the backing map under
+    // a read lock for the block offset
+    final List<ReentrantReadWriteLock> locks = new ArrayList<>();
+    LOG.debug("Notifying caching completed for file {}, with total blocks {}, and data blocks {}",
+      fileName, totalBlockCount, dataBlockCount);
+    try {
+      final MutableInt count = new MutableInt();
+      LOG.debug("iterating over {} entries in the backing map", backingMap.size());
+      backingMap.entrySet().stream().forEach(entry -> {
+        if (
+          entry.getKey().getHfileName().equals(fileName.getName())
+            && entry.getKey().getBlockType().equals(BlockType.DATA)
+        ) {
+          long offsetToLock = entry.getValue().offset();
+          LOG.debug("found block {} in the backing map. Acquiring read lock for offset {}",
+            entry.getKey(), offsetToLock);
+          ReentrantReadWriteLock lock = offsetLock.getLock(offsetToLock);
+          lock.readLock().lock();
+          locks.add(lock);
+          // rechecks the given key is still there (no eviction happened before the lock acquired)
+          if (backingMap.containsKey(entry.getKey())) {
+            count.increment();
+          } else {
+            lock.readLock().unlock();
+            locks.remove(lock);
+            LOG.debug("found block {}, but when locked and tried to count, it was gone.");
+          }
+        }
+      });
+      int metaCount = totalBlockCount - dataBlockCount;
+      // BucketCache would only have data blocks
+      if (dataBlockCount == count.getValue()) {
+        LOG.debug("File {} has now been fully cached.", fileName);
+        fileCacheCompleted(fileName, size);
+      } else {
+        LOG.debug(
+          "Prefetch executor completed for {}, but only {} data blocks were cached. "
+            + "Total data blocks for file: {}. "
+            + "Checking for blocks pending cache in cache writer queue.",
+          fileName, count.getValue(), dataBlockCount);
+        if (ramCache.hasBlocksForFile(fileName.getName())) {
+          for (ReentrantReadWriteLock lock : locks) {
+            lock.readLock().unlock();
+          }
+          LOG.debug("There are still blocks pending caching for file {}. Will sleep 100ms "
+            + "and try the verification again.", fileName.getName());
+          Thread.sleep(100);
+          notifyFileCachingCompleted(fileName, totalBlockCount, dataBlockCount, size);
+        } else
+          if ((getAllCacheKeysForFile(fileName.getName()).size() - metaCount) == dataBlockCount) {
+            LOG.debug("We counted {} data blocks, expected was {}, there was no more pending in "
+              + "the cache write queue but we now found that total cached blocks for file {} "
+              + "is equal to data block count.", count, dataBlockCount, fileName.getName());
+            fileCacheCompleted(fileName, size);
+          } else {
+            LOG.info("We found only {} data blocks cached from a total of {} for file {}, "
+              + "but no blocks pending caching. Maybe cache is full or evictions "
+              + "happened concurrently to cache prefetch.", count, dataBlockCount, fileName);
+          }
+      }
+    } catch (InterruptedException e) {
+      throw new RuntimeException(e);
+    } finally {
+      for (ReentrantReadWriteLock lock : locks) {
+        lock.readLock().unlock();
+      }
+    }
+  }
+
+  @Override
+  public void notifyFileBlockEvicted(String fileName) {
+    removeFileFromPrefetch(fileName);
+  }
+
+  @Override
+  public Optional<Boolean> blockFitsIntoTheCache(HFileBlock block) {
+    long currentUsed = bucketAllocator.getUsedSize();
+    boolean result = (currentUsed + block.getOnDiskSizeWithHeader()) < acceptableSize();
+    return Optional.of(result);
+  }
+
+  @Override
+  public Optional<Boolean> shouldCacheFile(String fileName) {
+    // if we don't have the file in fullyCachedFiles, we should cache it
+    return Optional.of(!fullyCachedFiles.containsKey(fileName));
+  }
+
+  @Override
+  public Optional<Boolean> isAlreadyCached(BlockCacheKey key) {
+    return Optional.of(getBackingMap().containsKey(key));
+  }
+
+  @Override
+  public Optional<Integer> getBlockSize(BlockCacheKey key) {
+    BucketEntry entry = backingMap.get(key);
+    if (entry == null) {
+      return Optional.empty();
+    } else {
+      return Optional.of(entry.getOnDiskSizeWithHeader());
+    }
+
+  }
+
+  private void removeFileFromPrefetch(String hfileName) {
+    // Update the regionPrefetchedSizeMap before removing the file from prefetchCompleted
+    if (fullyCachedFiles.containsKey(hfileName)) {
+      Pair<String, Long> regionEntry = fullyCachedFiles.get(hfileName);
+      String regionEncodedName = regionEntry.getFirst();
+      long filePrefetchSize = regionEntry.getSecond();
+      LOG.debug("Removing file {} for region {}", hfileName, regionEncodedName);
+      regionCachedSizeMap.computeIfPresent(regionEncodedName, (rn, pf) -> pf - filePrefetchSize);
+      // If all the blocks for a region are evicted from the cache, remove the entry for that region
+      if (
+        regionCachedSizeMap.containsKey(regionEncodedName)
+          && regionCachedSizeMap.get(regionEncodedName) == 0
+      ) {
+        regionCachedSizeMap.remove(regionEncodedName);
+      }
+    }
+    fullyCachedFiles.remove(hfileName);
+  }
+
+  public void fileCacheCompleted(Path filePath, long size) {
+    Pair<String, Long> pair = new Pair<>();
+    // sets the region name
+    String regionName = filePath.getParent().getParent().getName();
+    pair.setFirst(regionName);
+    pair.setSecond(size);
+    fullyCachedFiles.put(filePath.getName(), pair);
+    regionCachedSizeMap.merge(regionName, size, (oldpf, fileSize) -> oldpf + fileSize);
+  }
+
 }
