@@ -24,8 +24,11 @@ import io.opentelemetry.context.Scope;
 import java.io.IOException;
 import java.net.InetAddress;
 import java.nio.ByteBuffer;
+import java.security.cert.X509Certificate;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.List;
+import java.util.Map;
 import java.util.Optional;
 import java.util.concurrent.atomic.AtomicInteger;
 import org.apache.hadoop.hbase.CellScanner;
@@ -43,12 +46,14 @@ import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.apache.hadoop.util.StringUtils;
 import org.apache.yetus.audience.InterfaceAudience;
 
+import org.apache.hbase.thirdparty.com.google.common.collect.Maps;
 import org.apache.hbase.thirdparty.com.google.protobuf.BlockingService;
 import org.apache.hbase.thirdparty.com.google.protobuf.CodedOutputStream;
 import org.apache.hbase.thirdparty.com.google.protobuf.Descriptors.MethodDescriptor;
 import org.apache.hbase.thirdparty.com.google.protobuf.Message;
 
 import org.apache.hadoop.hbase.shaded.protobuf.ProtobufUtil;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.HBaseProtos;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.HBaseProtos.VersionInfo;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.RPCProtos.CellBlockMeta;
 import org.apache.hadoop.hbase.shaded.protobuf.generated.RPCProtos.ExceptionResponse;
@@ -92,13 +97,16 @@ public abstract class ServerCall<T extends ServerRpcConnection> implements RpcCa
 
   protected final User user;
   protected final InetAddress remoteAddress;
+  protected final X509Certificate[] clientCertificateChain;
   protected RpcCallback rpcCallback;
 
   private long responseCellSize = 0;
   private long responseBlockSize = 0;
+  private long fsReadTimeMillis = 0;
   // cumulative size of serialized exceptions
   private long exceptionSize = 0;
   private final boolean retryImmediatelySupported;
+  private volatile Map<String, byte[]> requestAttributes;
 
   // This is a dirty hack to address HBASE-22539. The highest bit is for rpc ref and cleanup, and
   // the rest of the bits are for WAL reference count. We can only call release if all of them are
@@ -130,9 +138,11 @@ public abstract class ServerCall<T extends ServerRpcConnection> implements RpcCa
     if (connection != null) {
       this.user = connection.user;
       this.retryImmediatelySupported = connection.retryImmediatelySupported;
+      this.clientCertificateChain = connection.clientCertificateChain;
     } else {
       this.user = null;
       this.retryImmediatelySupported = false;
+      this.clientCertificateChain = null;
     }
     this.remoteAddress = remoteAddress;
     this.timeout = timeout;
@@ -207,6 +217,41 @@ public abstract class ServerCall<T extends ServerRpcConnection> implements RpcCa
     return this.header;
   }
 
+  @Override
+  public Map<String, byte[]> getConnectionAttributes() {
+    return this.connection.connectionAttributes;
+  }
+
+  @Override
+  public Map<String, byte[]> getRequestAttributes() {
+    if (this.requestAttributes == null) {
+      if (header.getAttributeList().isEmpty()) {
+        this.requestAttributes = Collections.emptyMap();
+      } else {
+        Map<String, byte[]> requestAttributes =
+          Maps.newHashMapWithExpectedSize(header.getAttributeList().size());
+        for (HBaseProtos.NameBytesPair nameBytesPair : header.getAttributeList()) {
+          requestAttributes.put(nameBytesPair.getName(), nameBytesPair.getValue().toByteArray());
+        }
+        this.requestAttributes = requestAttributes;
+      }
+    }
+    return this.requestAttributes;
+  }
+
+  @Override
+  public byte[] getRequestAttribute(String key) {
+    if (this.requestAttributes == null) {
+      for (HBaseProtos.NameBytesPair nameBytesPair : header.getAttributeList()) {
+        if (nameBytesPair.getName().equals(key)) {
+          return nameBytesPair.getValue().toByteArray();
+        }
+      }
+      return null;
+    }
+    return this.requestAttributes.get(key);
+  }
+
   @Override
   public int getPriority() {
     return this.header.getPriority();
@@ -294,7 +339,6 @@ public abstract class ServerCall<T extends ServerRpcConnection> implements RpcCa
       bc = new BufferChain(responseBufs);
     } catch (IOException e) {
       RpcServer.LOG.warn("Exception while creating response " + e);
-      bc = createFallbackErrorResponse(e);
     }
     this.response = bc;
     // Once a response message is created and set to this.response, this Call can be treated as
@@ -331,32 +375,6 @@ public abstract class ServerCall<T extends ServerRpcConnection> implements RpcCa
     headerBuilder.setException(exceptionBuilder.build());
   }
 
-  /*
-   * Creates a fallback error response when the primary response creation fails. This method is
-   * invoked as a last resort when an IOException occurs during the normal response creation
-   * process. It attempts to create a minimal error response containing only the error information,
-   * without any cell blocks or additional data. The purpose is to ensure that the client receives
-   * some indication of the failure rather than experiencing a silent connection drop. This provides
-   * better error handling on the client side.
-   */
-  private BufferChain createFallbackErrorResponse(IOException originalException) {
-    try {
-      ResponseHeader.Builder headerBuilder = ResponseHeader.newBuilder();
-      headerBuilder.setCallId(this.id);
-      String responseErrorMsg =
-        "Failed to create response due to: " + originalException.getMessage();
-      setExceptionResponse(originalException, responseErrorMsg, headerBuilder);
-      Message header = headerBuilder.build();
-      ByteBuffer headerBuf = createHeaderAndMessageBytes(null, header, 0, null);
-      this.isError = true;
-      return new BufferChain(new ByteBuffer[] { headerBuf });
-    } catch (IOException e) {
-      RpcServer.LOG.error("Failed to create error response for client, connection may be dropped",
-        e);
-      return null;
-    }
-  }
-
   static ByteBuffer createHeaderAndMessageBytes(Message result, Message header, int cellBlockSize,
     List<ByteBuffer> cellBlock) throws IOException {
     // Organize the response as a set of bytebuffers rather than collect it all together inside
@@ -426,37 +444,6 @@ public abstract class ServerCall<T extends ServerRpcConnection> implements RpcCa
     return pbBuf;
   }
 
-  protected BufferChain wrapWithSasl(BufferChain bc) throws IOException {
-    if (!this.connection.useSasl) {
-      return bc;
-    }
-    // Looks like no way around this; saslserver wants a byte array. I have to make it one.
-    // THIS IS A BIG UGLY COPY.
-    byte[] responseBytes = bc.getBytes();
-    byte[] token;
-    // synchronization may be needed since there can be multiple Handler
-    // threads using saslServer or Crypto AES to wrap responses.
-    if (connection.useCryptoAesWrap) {
-      // wrap with Crypto AES
-      synchronized (connection.cryptoAES) {
-        token = connection.cryptoAES.wrap(responseBytes, 0, responseBytes.length);
-      }
-    } else {
-      synchronized (connection.saslServer) {
-        token = connection.saslServer.wrap(responseBytes, 0, responseBytes.length);
-      }
-    }
-    if (RpcServer.LOG.isTraceEnabled()) {
-      RpcServer.LOG
-        .trace("Adding saslServer wrapped token of size " + token.length + " as call response.");
-    }
-
-    ByteBuffer[] responseBufs = new ByteBuffer[2];
-    responseBufs[0] = ByteBuffer.wrap(Bytes.toBytes(token.length));
-    responseBufs[1] = ByteBuffer.wrap(token);
-    return new BufferChain(responseBufs);
-  }
-
   @Override
   public long disconnectSince() {
     if (!this.connection.isConnectionOpen()) {
@@ -482,12 +469,12 @@ public abstract class ServerCall<T extends ServerRpcConnection> implements RpcCa
   }
 
   @Override
-  public long getResponseBlockSize() {
+  public long getBlockBytesScanned() {
     return responseBlockSize;
   }
 
   @Override
-  public void incrementResponseBlockSize(long blockSize) {
+  public void incrementBlockBytesScanned(long blockSize) {
     responseBlockSize += blockSize;
   }
 
@@ -516,6 +503,11 @@ public abstract class ServerCall<T extends ServerRpcConnection> implements RpcCa
     return Optional.ofNullable(user);
   }
 
+  @Override
+  public Optional<X509Certificate[]> getClientCertificateChain() {
+    return Optional.ofNullable(clientCertificateChain);
+  }
+
   @Override
   public InetAddress getRemoteAddress() {
     return remoteAddress;
@@ -583,21 +575,7 @@ public abstract class ServerCall<T extends ServerRpcConnection> implements RpcCa
 
   @Override
   public synchronized BufferChain getResponse() {
-    if (connection.useWrap) {
-      /*
-       * wrapping result with SASL as the last step just before sending it out, so every message
-       * must have the right increasing sequence number
-       */
-      try {
-        return wrapWithSasl(response);
-      } catch (IOException e) {
-        /* it is exactly the same what setResponse() does */
-        RpcServer.LOG.warn("Exception while creating response " + e);
-        return null;
-      }
-    } else {
-      return response;
-    }
+    return response;
   }
 
   @RestrictedApi(explanation = "Should only be called in tests", link = "",
@@ -605,4 +583,14 @@ public abstract class ServerCall<T extends ServerRpcConnection> implements RpcCa
   public synchronized RpcCallback getCallBack() {
     return this.rpcCallback;
   }
+
+  @Override
+  public void updateFsReadTime(long latencyMillis) {
+    fsReadTimeMillis += latencyMillis;
+  }
+
+  @Override
+  public long getFsReadTime() {
+    return fsReadTimeMillis;
+  }
 }
