@@ -51,7 +51,6 @@ import org.apache.hadoop.hbase.master.procedure.MasterProcedureEnv;
 import org.apache.hadoop.hbase.master.procedure.MasterProcedureUtil;
 import org.apache.hadoop.hbase.procedure2.ProcedureMetrics;
 import org.apache.hadoop.hbase.procedure2.ProcedureStateSerializer;
-import org.apache.hadoop.hbase.quotas.MasterQuotaManager;
 import org.apache.hadoop.hbase.quotas.QuotaExceededException;
 import org.apache.hadoop.hbase.regionserver.HRegionFileSystem;
 import org.apache.hadoop.hbase.regionserver.HStoreFile;
@@ -445,24 +444,11 @@ public class MergeTableRegionsProcedure
   private boolean prepareMergeRegion(final MasterProcedureEnv env) throws IOException {
     // Fail if we are taking snapshot for the given table
     TableName tn = regionsToMerge[0].getTable();
-    if (env.getMasterServices().getSnapshotManager().isTakingSnapshot(tn)) {
+    if (env.getMasterServices().getSnapshotManager().isTableTakingAnySnapshot(tn)) {
       throw new MergeRegionException("Skip merging regions "
         + RegionInfo.getShortNameToLog(regionsToMerge) + ", because we are snapshotting " + tn);
     }
 
-    /*
-     * Sometimes a ModifyTableProcedure has edited a table descriptor to change the number of region
-     * replicas for a table, but it has not yet opened/closed the new replicas. The
-     * ModifyTableProcedure assumes that nobody else will do the opening/closing of the new
-     * replicas, but a concurrent MergeTableRegionProcedure would violate that assumption.
-     */
-    if (isTableModificationInProgress(env)) {
-      setFailure(getClass().getSimpleName(),
-        new IOException("Skip merging regions " + RegionInfo.getShortNameToLog(regionsToMerge)
-          + ", because there is an active procedure that is modifying the table " + tn));
-      return false;
-    }
-
     // Mostly this check is not used because we already check the switch before submit a merge
     // procedure. Just for safe, check the switch again. This procedure can be rollbacked if
     // the switch was set to false after submit.
@@ -553,10 +539,7 @@ public class MergeTableRegionsProcedure
     }
     // TODO: Clean up split and merge. Currently all over the place.
     try {
-      MasterQuotaManager masterQuotaManager = env.getMasterServices().getMasterQuotaManager();
-      if (masterQuotaManager != null) {
-        masterQuotaManager.onRegionMerged(this.mergedRegion);
-      }
+      env.getMasterServices().getMasterQuotaManager().onRegionMerged(this.mergedRegion);
     } catch (QuotaExceededException e) {
       // TODO: why is this here? merge requests can be submitted by actors other than the normalizer
       env.getMasterServices().getRegionNormalizerManager()
@@ -671,8 +654,8 @@ public class MergeTableRegionsProcedure
    **/
   private void rollbackCloseRegionsForMerge(MasterProcedureEnv env) throws IOException {
     // At this point we should check if region was actually closed. If it was not closed then we
-    // don't need to reopen the region and we can just change the regionNode state to OPEN.
-    // if it is already closed then we need to do a reopen of region
+    // don't need to repoen the region and we can just change the regionNode state to OPEN.
+    // if it is alredy closed then we need to do a reopen of region
     List<RegionInfo> toAssign = new ArrayList<>();
     for (RegionInfo rinfo : regionsToMerge) {
       RegionStateNode regionStateNode =
