@@ -47,7 +47,7 @@ class TableQueue extends Queue<TableName> {
   /**
    * @param proc must not be null
    */
-  static boolean requireTableExclusiveLock(TableProcedureInterface proc) {
+  private static boolean requireTableExclusiveLock(TableProcedureInterface proc) {
     switch (proc.getTableOperationType()) {
       case CREATE:
       case DELETE:
@@ -58,6 +58,8 @@ class TableQueue extends Queue<TableName> {
         // we allow concurrent edit on the NS table
         return !proc.getTableName().equals(TableName.NAMESPACE_TABLE_NAME);
       case READ:
+      case FLUSH:
+      case SNAPSHOT:
         return false;
       // region operations are using the shared-lock on the table
       // and then they will grab an xlock on the region.
@@ -68,6 +70,8 @@ class TableQueue extends Queue<TableName> {
       case REGION_EDIT:
       case REGION_GC:
       case MERGED_REGIONS_GC:
+      case REGION_SNAPSHOT:
+      case REGION_TRUNCATE:
         return false;
       default:
         break;
