@@ -21,9 +21,7 @@ import static org.apache.hadoop.hbase.HConstants.DEFAULT_HBASE_RPC_TIMEOUT;
 import static org.apache.hadoop.hbase.HConstants.HBASE_RPC_TIMEOUT_KEY;
 
 import java.io.IOException;
-import java.util.HashSet;
 import java.util.List;
-import java.util.Set;
 import java.util.concurrent.CancellationException;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileSystem;
@@ -50,9 +48,7 @@ import org.apache.hadoop.hbase.procedure2.LockType;
 import org.apache.hadoop.hbase.snapshot.ClientSnapshotDescriptionUtils;
 import org.apache.hadoop.hbase.snapshot.SnapshotDescriptionUtils;
 import org.apache.hadoop.hbase.snapshot.SnapshotManifest;
-import org.apache.hadoop.hbase.snapshot.SnapshotTTLExpiredException;
 import org.apache.hadoop.hbase.util.CommonFSUtils;
-import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.apache.hadoop.hbase.util.Pair;
 import org.apache.yetus.audience.InterfaceAudience;
 import org.apache.zookeeper.KeeperException;
@@ -90,7 +86,7 @@ public abstract class TakeSnapshotHandler extends EventHandler
   protected final Path workingDir;
   private final MasterSnapshotVerifier verifier;
   protected final ForeignExceptionDispatcher monitor;
-  private LockManager.MasterLock tableLock;
+  private final LockManager.MasterLock tableLock;
   protected final MonitoredTask status;
   protected final TableName snapshotTable;
   protected final SnapshotManifest snapshotManifest;
@@ -169,30 +165,17 @@ public abstract class TakeSnapshotHandler extends EventHandler
     if (this.tableLock.tryAcquire(this.lockAcquireTimeoutMs)) {
       try {
         this.htd = loadTableDescriptor(); // check that .tableinfo is present
-        if (downgradeToSharedTableLock()) {
-          // release the exclusive lock and hold the shared lock instead
-          this.tableLock.release();
-          this.tableLock = master.getLockManager().createMasterLock(snapshotTable, LockType.SHARED,
-            this.getClass().getName() + ": take snapshot " + snapshot.getName());
-          if (!this.tableLock.tryAcquire(this.lockAcquireTimeoutMs)) {
-            throwLockNotAcquiredException();
-          }
-        }
       } catch (Exception e) {
         this.tableLock.release();
         throw e;
       }
     } else {
-      throwLockNotAcquiredException();
+      LOG.error("Master lock could not be acquired in {} ms", lockAcquireTimeoutMs);
+      throw new DoNotRetryIOException("Master lock could not be acquired");
     }
     return this;
   }
 
-  private void throwLockNotAcquiredException() throws DoNotRetryIOException {
-    LOG.error("Master lock could not be acquired in {} ms", lockAcquireTimeoutMs);
-    throw new DoNotRetryIOException("Master lock could not be acquired");
-  }
-
   /**
    * Execute the core common portions of taking a snapshot. The {@link #snapshotRegions(List)} call
    * should get implemented for each snapshot flavor.
@@ -207,6 +190,18 @@ public abstract class TakeSnapshotHandler extends EventHandler
     MasterLock tableLockToRelease = this.tableLock;
     status.setStatus(msg);
     try {
+      if (downgradeToSharedTableLock()) {
+        // release the exclusive lock and hold the shared lock instead
+        tableLockToRelease = master.getLockManager().createMasterLock(snapshotTable,
+          LockType.SHARED, this.getClass().getName() + ": take snapshot " + snapshot.getName());
+        tableLock.release();
+        boolean isTableLockAcquired = tableLockToRelease.tryAcquire(this.lockAcquireTimeoutMs);
+        if (!isTableLockAcquired) {
+          LOG.error("Could not acquire shared lock on table {} in {} ms", snapshotTable,
+            lockAcquireTimeoutMs);
+          throw new IOException("Could not acquire shared lock on table " + snapshotTable);
+        }
+      }
       // If regions move after this meta scan, the region specific snapshot should fail, triggering
       // an external exception that gets captured here.
 
@@ -222,31 +217,13 @@ public abstract class TakeSnapshotHandler extends EventHandler
       snapshotRegions(regionsAndLocations);
       monitor.rethrowException();
 
-      // extract each pair to separate lists
-      Set<String> serverNames = new HashSet<>();
-      for (Pair<RegionInfo, ServerName> p : regionsAndLocations) {
-        if (p != null && p.getFirst() != null && p.getSecond() != null) {
-          RegionInfo hri = p.getFirst();
-          if (hri.isOffline() && (hri.isSplit() || hri.isSplitParent())) continue;
-          serverNames.add(p.getSecond().toString());
-        }
-      }
-
       // flush the in-memory state, and write the single manifest
       status.setStatus("Consolidate snapshot: " + snapshot.getName());
       snapshotManifest.consolidate();
 
       // verify the snapshot is valid
       status.setStatus("Verifying snapshot: " + snapshot.getName());
-      verifier.verifySnapshot(this.workingDir, serverNames);
-
-      // HBASE-29296 check snapshot is not expired
-      if (
-        SnapshotDescriptionUtils.isExpiredSnapshot(snapshot.getTtl(), snapshot.getCreationTime(),
-          EnvironmentEdgeManager.currentTime())
-      ) {
-        throw new SnapshotTTLExpiredException(ProtobufUtil.createSnapshotDesc(snapshot));
-      }
+      verifier.verifySnapshot(workingDir, true);
 
       // complete the snapshot, atomically moving from tmp to .snapshot dir.
       SnapshotDescriptionUtils.completeSnapshot(this.snapshotDir, this.workingDir, this.rootFs,
@@ -275,11 +252,11 @@ public abstract class TakeSnapshotHandler extends EventHandler
       try {
         // if the working dir is still present, the snapshot has failed. it is present we delete
         // it.
-        if (workingDirFs.exists(workingDir) && !workingDirFs.delete(workingDir, true)) {
-          LOG.error("Couldn't delete snapshot working directory: {}", workingDir);
+        if (!workingDirFs.delete(workingDir, true)) {
+          LOG.error("Couldn't delete snapshot working directory:" + workingDir);
         }
       } catch (IOException e) {
-        LOG.error("Couldn't get or delete snapshot working directory: {}", workingDir, e);
+        LOG.error("Couldn't delete snapshot working directory:" + workingDir);
       }
       if (LOG.isDebugEnabled()) {
         LOG.debug("Table snapshot journal : \n" + status.prettyPrintJournal());
