@@ -70,6 +70,11 @@ class NoopQuotaLimiter implements QuotaLimiter {
     throw new UnsupportedOperationException();
   }
 
+  @Override
+  public long getReadLimit() {
+    return Long.MAX_VALUE;
+  }
+
   @Override
   public String toString() {
     return "NoopQuotaLimiter";
