@@ -82,6 +82,7 @@ public abstract class Compactor<T extends CellSink> {
   protected final Configuration conf;
   protected final HStore store;
   protected final int compactionKVMax;
+  protected final long compactScannerSizeLimit;
   protected final Compression.Algorithm majorCompactionCompression;
   protected final Compression.Algorithm minorCompactionCompression;
 
@@ -108,6 +109,8 @@ public abstract class Compactor<T extends CellSink> {
     this.store = store;
     this.compactionKVMax =
       this.conf.getInt(HConstants.COMPACTION_KV_MAX, HConstants.COMPACTION_KV_MAX_DEFAULT);
+    this.compactScannerSizeLimit = this.conf.getLong(HConstants.COMPACTION_SCANNER_SIZE_MAX,
+      HConstants.COMPACTION_SCANNER_SIZE_MAX_DEFAULT);
     this.majorCompactionCompression = (store.getColumnFamilyDescriptor() == null)
       ? Compression.Algorithm.NONE
       : store.getColumnFamilyDescriptor().getMajorCompactionCompressionType();
@@ -428,8 +431,10 @@ public abstract class Compactor<T extends CellSink> {
     String compactionName = ThroughputControlUtil.getNameForThrottling(store, "compaction");
     long now = 0;
     boolean hasMore;
-    ScannerContext scannerContext =
-      ScannerContext.newBuilder().setBatchLimit(compactionKVMax).build();
+    ScannerContext scannerContext = ScannerContext.newBuilder().setBatchLimit(compactionKVMax)
+      .setSizeLimit(ScannerContext.LimitScope.BETWEEN_CELLS, Long.MAX_VALUE, Long.MAX_VALUE,
+        compactScannerSizeLimit)
+      .build();
 
     throughputController.start(compactionName);
     Shipper shipper = (scanner instanceof Shipper) ? (Shipper) scanner : null;
@@ -458,6 +463,7 @@ public abstract class Compactor<T extends CellSink> {
             lastCleanCell = null;
             lastCleanCellSeqId = 0;
           }
+          writer.append(c);
           int len = c.getSerializedSize();
           ++progress.currentCompactedKVs;
           progress.totalCompactedSize += len;
@@ -471,7 +477,6 @@ public abstract class Compactor<T extends CellSink> {
             return false;
           }
         }
-        writer.appendAll(cells);
         if (shipper != null && bytesWrittenProgressForShippedCall > shippedCallSizeLimit) {
           if (lastCleanCell != null) {
             // HBASE-16931, set back sequence id to avoid affecting scan order unexpectedly.
