@@ -63,19 +63,22 @@ public class UnassignRegionHandler extends EventHandler {
 
   private final RetryCounter retryCounter;
 
-  // active time of the master that sent this unassign request, used for fencing
-  private final long initiatingMasterActiveTime;
+  private boolean evictCache;
 
   public UnassignRegionHandler(HRegionServer server, String encodedName, long closeProcId,
-    boolean abort, @Nullable ServerName destination, EventType eventType,
-    long initiatingMasterActiveTime) {
+    boolean abort, @Nullable ServerName destination, EventType eventType) {
+    this(server, encodedName, closeProcId, abort, destination, eventType, false);
+  }
+
+  public UnassignRegionHandler(HRegionServer server, String encodedName, long closeProcId,
+    boolean abort, @Nullable ServerName destination, EventType eventType, boolean evictCache) {
     super(server, eventType);
     this.encodedName = encodedName;
     this.closeProcId = closeProcId;
     this.abort = abort;
     this.destination = destination;
     this.retryCounter = HandlerUtil.getRetryCounter();
-    this.initiatingMasterActiveTime = initiatingMasterActiveTime;
+    this.evictCache = evictCache;
   }
 
   private HRegionServer getServer() {
@@ -122,6 +125,12 @@ public class UnassignRegionHandler extends EventHandler {
       // abort the RS...
       region.getCoprocessorHost().preClose(abort);
     }
+    // This should be true only in the case of splits/merges closing the parent regions, as
+    // there's no point on keep blocks for those region files. As hbase.rs.evictblocksonclose is
+    // false by default we don't bother overriding it if evictCache is false.
+    if (evictCache) {
+      region.getStores().forEach(s -> s.getCacheConfig().setEvictOnClose(true));
+    }
     if (region.close(abort) == null) {
       // XXX: Is this still possible? The old comment says about split, but now split is done at
       // master side, so...
@@ -144,7 +153,7 @@ public class UnassignRegionHandler extends EventHandler {
     }
     if (
       !rs.reportRegionStateTransition(new RegionStateTransitionContext(TransitionCode.CLOSED,
-        HConstants.NO_SEQNUM, closeProcId, -1, region.getRegionInfo(), initiatingMasterActiveTime))
+        HConstants.NO_SEQNUM, closeProcId, -1, region.getRegionInfo()))
     ) {
       throw new IOException("Failed to report close to master: " + regionName);
     }
@@ -164,8 +173,7 @@ public class UnassignRegionHandler extends EventHandler {
   }
 
   public static UnassignRegionHandler create(HRegionServer server, String encodedName,
-    long closeProcId, boolean abort, @Nullable ServerName destination,
-    long initiatingMasterActiveTime) {
+    long closeProcId, boolean abort, @Nullable ServerName destination, boolean evictCache) {
     // Just try our best to determine whether it is for closing meta. It is not the end of the world
     // if we put the handler into a wrong executor.
     Region region = server.getRegion(encodedName);
@@ -173,6 +181,6 @@ public class UnassignRegionHandler extends EventHandler {
       ? EventType.M_RS_CLOSE_META
       : EventType.M_RS_CLOSE_REGION;
     return new UnassignRegionHandler(server, encodedName, closeProcId, abort, destination,
-      eventType, initiatingMasterActiveTime);
+      eventType, evictCache);
   }
 }
