@@ -46,6 +46,10 @@ import org.apache.hadoop.hbase.shaded.protobuf.generated.WALProtos.RegionEventDe
  * that came in as a single transaction. All the edits for a given transaction are written out as a
  * single record, in PB format, followed (optionally) by Cells written via the WALCellEncoder.
  * <p>
+ * This class is LimitedPrivate for CPs to read-only. The {@link #add} methods are classified as
+ * private methods, not for use by CPs.
+ * </p>
+ * <p>
  * A particular WALEdit 'type' is the 'meta' type used to mark key operational events in the WAL
  * such as compaction, flush, or region open. These meta types do not traverse hbase memstores. They
  * are edits made by the hbase system rather than edit data submitted by clients. They only show in
@@ -69,6 +73,7 @@ import org.apache.hadoop.hbase.shaded.protobuf.generated.WALProtos.RegionEventDe
  * file. See the hand-off in FSWALEntry Constructor.
  * @see WALKey
  */
+// TODO: Do not expose this class to Coprocessors. It has set methods. A CP might meddle.
 @InterfaceAudience.LimitedPrivate({ HBaseInterfaceAudience.REPLICATION,
   HBaseInterfaceAudience.COPROC })
 public class WALEdit implements HeapSize {
@@ -127,6 +132,22 @@ public class WALEdit implements HeapSize {
   @InterfaceAudience.Private
   public static final byte[] BULK_LOAD = Bytes.toBytes("HBASE::BULK_LOAD");
 
+  /**
+   * Periodically {@link org.apache.hadoop.hbase.replication.regionserver.ReplicationMarkerChore}
+   * will create marker edits with family as {@link WALEdit#METAFAMILY} and
+   * {@link WALEdit#REPLICATION_MARKER} as qualifier and an empty value.
+   * org.apache.hadoop.hbase.replication.regionserver.ReplicationSourceWALReader will populate the
+   * Replication Marker edit with region_server_name, wal_name and wal_offset encoded in
+   * {@link org.apache.hadoop.hbase.shaded.protobuf.generated.WALProtos.ReplicationMarkerDescriptor}
+   * object. {@link org.apache.hadoop.hbase.replication.regionserver.Replication} will change the
+   * REPLICATION_SCOPE for this edit to GLOBAL so that it can replicate. On the sink cluster,
+   * {@link org.apache.hadoop.hbase.replication.regionserver.ReplicationSink} will convert the
+   * ReplicationMarkerDescriptor into a Put mutation to REPLICATION_SINK_TRACKER_TABLE_NAME_STR
+   * table.
+   */
+  @InterfaceAudience.Private
+  public static final byte[] REPLICATION_MARKER = Bytes.toBytes("HBASE::REPLICATION_MARKER");
+
   private final transient boolean replay;
 
   private ArrayList<Cell> cells;
@@ -219,11 +240,13 @@ public class WALEdit implements HeapSize {
     return this.replay;
   }
 
+  @InterfaceAudience.Private
   public WALEdit add(Cell cell, byte[] family) {
     getOrCreateFamilies().add(family);
     return addCell(cell);
   }
 
+  @InterfaceAudience.Private
   public WALEdit add(Cell cell) {
     // We clone Family each time we add a Cell. Expensive but safe. For CPU savings, use
     // add(Map) or add(Cell, family).
@@ -247,6 +270,7 @@ public class WALEdit implements HeapSize {
    * that nothing else depends on the contents being immutable.
    * @param cells the list of cells that this WALEdit now contains.
    */
+  @InterfaceAudience.Private
   // Used by replay.
   public void setCells(ArrayList<Cell> cells) {
     this.cells = cells;
@@ -446,4 +470,28 @@ public class WALEdit implements HeapSize {
     this.cells.add(cell);
     return this;
   }
+
+  /**
+   * Creates a replication tracker edit with {@link #METAFAMILY} family and
+   * {@link #REPLICATION_MARKER} qualifier and has null value.
+   * @param rowKey    rowkey
+   * @param timestamp timestamp
+   */
+  public static WALEdit createReplicationMarkerEdit(byte[] rowKey, long timestamp) {
+    KeyValue kv =
+      new KeyValue(rowKey, METAFAMILY, REPLICATION_MARKER, timestamp, KeyValue.Type.Put);
+    return new WALEdit().add(kv);
+  }
+
+  /**
+   * Checks whether this edit is a replication marker edit.
+   * @param edit edit
+   * @return true if the cell within an edit has column = METAFAMILY and qualifier =
+   *         REPLICATION_MARKER, false otherwise
+   */
+  public static boolean isReplicationMarkerEdit(WALEdit edit) {
+    // Check just the first cell from the edit. ReplicationMarker edit will have only 1 cell.
+    return edit.getCells().size() == 1
+      && CellUtil.matchingColumn(edit.getCells().get(0), METAFAMILY, REPLICATION_MARKER);
+  }
 }
