@@ -58,19 +58,16 @@ import org.apache.hadoop.hbase.replication.ReplicationPeerConfig;
 import org.apache.hadoop.hbase.replication.ReplicationPeerDescription;
 import org.apache.hadoop.hbase.security.access.AccessChecker;
 import org.apache.hadoop.hbase.security.access.ZKPermissionWatcher;
-import org.apache.hadoop.hbase.util.EnvironmentEdgeManager;
 import org.apache.hadoop.hbase.zookeeper.ZKWatcher;
 
 public class MockNoopMasterServices implements MasterServices {
 
   private final Configuration conf;
   private final MetricsMaster metricsMaster;
-  private final long masterActiveTime;
 
   public MockNoopMasterServices(final Configuration conf) {
     this.conf = conf;
     this.metricsMaster = new MetricsMaster(new MetricsMasterWrapperImpl(mock(HMaster.class)));
-    this.masterActiveTime = EnvironmentEdgeManager.currentTime();
   }
 
   @Override
@@ -78,6 +75,12 @@ public class MockNoopMasterServices implements MasterServices {
     // no-op
   }
 
+  @Override
+  public long truncateRegion(RegionInfo regionInfo, long nonceGroup, long nonce)
+    throws IOException {
+    return 0;
+  }
+
   @Override
   public long createTable(final TableDescriptor desc, final byte[][] splitKeys,
     final long nonceGroup, final long nonce) throws IOException {
@@ -319,11 +322,6 @@ public class MockNoopMasterServices implements MasterServices {
     return true;
   }
 
-  @Override
-  public long getMasterActiveTime() {
-    return masterActiveTime;
-  }
-
   @Override
   public boolean isInitialized() {
     return false;
@@ -511,4 +509,20 @@ public class MockNoopMasterServices implements MasterServices {
     long nonceGroup, long nonce) throws IOException {
     return -1;
   }
+
+  @Override
+  public boolean replicationPeerModificationSwitch(boolean on) throws IOException {
+    return false;
+  }
+
+  @Override
+  public boolean isReplicationPeerModificationEnabled() {
+    return false;
+  }
+
+  @Override
+  public long flushTable(TableName tableName, List<byte[]> columnFamilies, long nonceGroup,
+    long nonce) throws IOException {
+    return 0;
+  }
 }
