@@ -25,6 +25,7 @@ import static org.junit.Assert.fail;
 
 import java.io.IOException;
 import java.util.List;
+import java.util.Optional;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hbase.Cell;
 import org.apache.hadoop.hbase.CellUtil;
@@ -40,18 +41,23 @@ import org.apache.hadoop.hbase.client.Get;
 import org.apache.hadoop.hbase.client.RegionInfo;
 import org.apache.hadoop.hbase.client.RegionReplicaUtil;
 import org.apache.hadoop.hbase.client.Result;
+import org.apache.hadoop.hbase.client.SnapshotDescription;
+import org.apache.hadoop.hbase.client.SnapshotType;
 import org.apache.hadoop.hbase.client.Table;
 import org.apache.hadoop.hbase.client.TableDescriptor;
-import org.apache.hadoop.hbase.client.TableDescriptorBuilder;
-import org.apache.hadoop.hbase.master.RegionState;
+import org.apache.hadoop.hbase.coprocessor.ObserverContext;
+import org.apache.hadoop.hbase.coprocessor.RegionCoprocessor;
+import org.apache.hadoop.hbase.coprocessor.RegionCoprocessorEnvironment;
+import org.apache.hadoop.hbase.coprocessor.RegionObserver;
 import org.apache.hadoop.hbase.master.procedure.MasterProcedureConstants;
 import org.apache.hadoop.hbase.master.procedure.MasterProcedureEnv;
 import org.apache.hadoop.hbase.master.procedure.MasterProcedureTestingUtility;
-import org.apache.hadoop.hbase.master.procedure.ModifyTableProcedure;
+import org.apache.hadoop.hbase.master.procedure.TestSnapshotProcedure;
 import org.apache.hadoop.hbase.procedure2.ProcedureExecutor;
 import org.apache.hadoop.hbase.procedure2.ProcedureMetrics;
 import org.apache.hadoop.hbase.procedure2.ProcedureTestingUtility;
 import org.apache.hadoop.hbase.regionserver.HRegion;
+import org.apache.hadoop.hbase.snapshot.SnapshotDescriptionUtils;
 import org.apache.hadoop.hbase.testclassification.MasterTests;
 import org.apache.hadoop.hbase.testclassification.MediumTests;
 import org.apache.hadoop.hbase.util.Bytes;
@@ -67,6 +73,9 @@ import org.junit.rules.TestName;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import org.apache.hadoop.hbase.shaded.protobuf.ProtobufUtil;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.SnapshotProtos;
+
 @Category({ MasterTests.class, MediumTests.class })
 public class TestSplitTableRegionProcedure {
 
@@ -104,7 +113,38 @@ public class TestSplitTableRegionProcedure {
     conf.setInt(MasterProcedureConstants.MASTER_PROCEDURE_THREADS, 1);
     conf.setLong(HConstants.MAJOR_COMPACTION_PERIOD, 0);
     conf.set("hbase.coprocessor.region.classes",
-      RegionServerHostingReplicaSlowOpenCoprocessor.class.getName());
+      RegionServerHostingReplicaSlowOpenCopro.class.getName());
+    conf.setInt("hbase.client.sync.wait.timeout.msec", 1500);
+  }
+
+  /**
+   * This copro is used to slow down opening of the replica regions.
+   */
+  public static class RegionServerHostingReplicaSlowOpenCopro
+    implements RegionCoprocessor, RegionObserver {
+    static int countForReplica = 0;
+    static boolean slowDownReplicaOpen = false;
+
+    @Override
+    public Optional<RegionObserver> getRegionObserver() {
+      return Optional.of(this);
+    }
+
+    @Override
+    public void preOpen(ObserverContext<RegionCoprocessorEnvironment> c) throws IOException {
+      int replicaId = c.getEnvironment().getRegion().getRegionInfo().getReplicaId();
+      if ((replicaId != RegionInfo.DEFAULT_REPLICA_ID) && (countForReplica == 0)) {
+        countForReplica++;
+        while (slowDownReplicaOpen) {
+          LOG.info("Slow down replica region open a bit");
+          try {
+            Thread.sleep(100);
+          } catch (InterruptedException ie) {
+            // Ingore
+          }
+        }
+      }
+    }
   }
 
   @BeforeClass
@@ -149,13 +189,15 @@ public class TestSplitTableRegionProcedure {
     final TableName tableName = TableName.valueOf(name.getMethodName());
     final ProcedureExecutor<MasterProcedureEnv> procExec = getMasterProcedureExecutor();
 
-    RegionServerHostingReplicaSlowOpenCoprocessor.slowDownReplicaOpen = true;
+    RegionServerHostingReplicaSlowOpenCopro.slowDownReplicaOpen = true;
     RegionInfo[] regions =
       MasterProcedureTestingUtility.createTable(procExec, tableName, null, columnFamilyName1);
 
-    TableDescriptor td = TableDescriptorBuilder.newBuilder(UTIL.getAdmin().getDescriptor(tableName))
-      .setRegionReplication(2).build();
-    procExec.submitProcedure(new ModifyTableProcedure(procExec.getEnvironment(), td));
+    try {
+      HBaseTestingUtility.setReplicas(UTIL.getAdmin(), tableName, 2);
+    } catch (IOException ioe) {
+
+    }
 
     // wait until the primary region is online.
     HBaseTestingUtility.await(2000, () -> {
@@ -179,7 +221,7 @@ public class TestSplitTableRegionProcedure {
     ProcedureTestingUtility.waitProcedure(procExec, procId);
 
     // Let replica parent region open.
-    RegionServerHostingReplicaSlowOpenCoprocessor.slowDownReplicaOpen = false;
+    RegionServerHostingReplicaSlowOpenCopro.slowDownReplicaOpen = false;
 
     // wait until the replica region is online.
     HBaseTestingUtility.await(2000, () -> {
@@ -187,10 +229,7 @@ public class TestSplitTableRegionProcedure {
         AssignmentManager am = UTIL.getHBaseCluster().getMaster().getAssignmentManager();
         if (am == null) return false;
         RegionInfo replicaRegion = RegionReplicaUtil.getRegionInfoForReplica(regions[0], 1);
-        if (
-          am.getRegionStates().getRegionState(replicaRegion) != null
-            && am.getRegionStates().getRegionState(replicaRegion).isOpened()
-        ) {
+        if (am.getRegionStates().getRegionState(replicaRegion).isOpened()) {
           return true;
         }
         return false;
@@ -203,11 +242,6 @@ public class TestSplitTableRegionProcedure {
     // There should not be any active OpenRegionProcedure
     procExec.getActiveProceduresNoCopy()
       .forEach(p -> assertFalse(p instanceof OpenRegionProcedure));
-
-    // Check that procedure rollback reverted parent region state to OPEN
-    AssignmentManager am = UTIL.getHBaseCluster().getMaster().getAssignmentManager();
-    RegionStateNode regionStateNode = am.getRegionStates().getRegionStateNode(regions[0]);
-    assertEquals(RegionState.State.OPEN, regionStateNode.getState());
   }
 
   @Test
@@ -524,29 +558,49 @@ public class TestSplitTableRegionProcedure {
   }
 
   @Test
-  public void testSplitDetectsModifyTableProcedure() throws Exception {
+  public void testSplitRegionWhileTakingSnapshot() throws Exception {
     final TableName tableName = TableName.valueOf(name.getMethodName());
     final ProcedureExecutor<MasterProcedureEnv> procExec = getMasterProcedureExecutor();
 
-    RegionInfo[] regions =
-      MasterProcedureTestingUtility.createTable(procExec, tableName, null, columnFamilyName1);
-    RegionServerHostingReplicaSlowOpenCoprocessor.slowDownReplicaOpen = true;
-    TableDescriptor td = TableDescriptorBuilder.newBuilder(UTIL.getAdmin().getDescriptor(tableName))
-      .setRegionReplication(2).build();
-    long modifyProcId =
-      procExec.submitProcedure(new ModifyTableProcedure(procExec.getEnvironment(), td));
+    RegionInfo[] regions = MasterProcedureTestingUtility.createTable(procExec, tableName, null,
+      columnFamilyName1, columnFamilyName2);
+    int splitRowNum = startRowNum + rowCount / 2;
+    byte[] splitKey = Bytes.toBytes("" + splitRowNum);
 
-    // Split region of the table, the SplitTableRegionProcedure will fail because there is a
-    // ModifyTableProcedure in progress
-    SplitTableRegionProcedure splitProcedure = new SplitTableRegionProcedure(
-      procExec.getEnvironment(), regions[0], HConstants.CATALOG_FAMILY);
-    long splitProcId = procExec.submitProcedure(splitProcedure);
-    ProcedureTestingUtility.waitProcedure(procExec, splitProcId);
-    ProcedureTestingUtility.assertProcFailed(procExec, splitProcId);
+    assertTrue("not able to find a splittable region", regions != null);
+    assertTrue("not able to find a splittable region", regions.length == 1);
+    ProcedureTestingUtility.waitNoProcedureRunning(procExec);
 
-    RegionServerHostingReplicaSlowOpenCoprocessor.slowDownReplicaOpen = false;
-    ProcedureTestingUtility.waitProcedure(procExec, modifyProcId);
-    ProcedureTestingUtility.assertProcNotFailed(procExec, modifyProcId);
+    // task snapshot
+    SnapshotDescription snapshot =
+      new SnapshotDescription("SnapshotProcedureTest", tableName, SnapshotType.FLUSH);
+    SnapshotProtos.SnapshotDescription snapshotProto =
+      ProtobufUtil.createHBaseProtosSnapshotDesc(snapshot);
+    snapshotProto = SnapshotDescriptionUtils.validate(snapshotProto,
+      UTIL.getHBaseCluster().getMaster().getConfiguration());
+    long snapshotProcId = procExec.submitProcedure(
+      new TestSnapshotProcedure.DelaySnapshotProcedure(procExec.getEnvironment(), snapshotProto));
+    UTIL.getHBaseCluster().getMaster().getSnapshotManager().registerSnapshotProcedure(snapshotProto,
+      snapshotProcId);
+
+    // collect AM metrics before test
+    collectAssignmentManagerMetrics();
+
+    // Split region of the table
+    long procId = procExec.submitProcedure(
+      new SplitTableRegionProcedure(procExec.getEnvironment(), regions[0], splitKey));
+    // Wait the completion
+    ProcedureTestingUtility.waitProcedure(procExec, procId);
+    ProcedureTestingUtility.waitProcedure(procExec, snapshotProcId);
+
+    ProcedureTestingUtility.assertProcFailed(procExec, procId);
+    ProcedureTestingUtility.assertProcNotFailed(procExec, snapshotProcId);
+
+    assertTrue(UTIL.getMiniHBaseCluster().getRegions(tableName).size() == 1);
+    assertTrue(UTIL.countRows(tableName) == 0);
+
+    assertEquals(splitSubmittedCount + 1, splitProcMetrics.getSubmittedCounter().getCount());
+    assertEquals(splitFailedCount + 1, splitProcMetrics.getFailedCounter().getCount());
   }
 
   private void deleteData(final TableName tableName, final int startDeleteRowNum)
