@@ -24,6 +24,7 @@ import static org.junit.Assert.fail;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.List;
 import java.util.Map.Entry;
 import java.util.Objects;
@@ -123,8 +124,13 @@ public class SpaceQuotaHelperForTests {
    * Returns the number of quotas defined in the HBase quota table.
    */
   long listNumDefinedQuotas(Connection conn) throws IOException {
-    try (QuotaRetriever scanner = new QuotaRetriever(conn)) {
+    QuotaRetriever scanner = QuotaRetriever.open(conn.getConfiguration());
+    try {
       return Iterables.size(scanner);
+    } finally {
+      if (scanner != null) {
+        scanner.close();
+      }
     }
   }
 
@@ -349,7 +355,8 @@ public class SpaceQuotaHelperForTests {
     Table table = conn.getTable(tn);
     final String bulkToken = new SecureBulkLoadClient(conf, table).prepareBulkLoad(conn);
     return new ClientServiceCallable<Void>(conn, tn, Bytes.toBytes("row"),
-      new RpcControllerFactory(conf).newController(), HConstants.PRIORITY_UNSET) {
+      new RpcControllerFactory(conf).newController(), HConstants.PRIORITY_UNSET,
+      Collections.emptyMap()) {
       @Override
       public Void rpcCall() throws Exception {
         SecureBulkLoadClient secureClient = null;
@@ -429,7 +436,8 @@ public class SpaceQuotaHelperForTests {
       waitForQuotaTable(conn);
     } else {
       // Or, clean up any quotas from previous test runs.
-      try (QuotaRetriever scanner = new QuotaRetriever(conn);) {
+      QuotaRetriever scanner = QuotaRetriever.open(conn.getConfiguration());
+      try {
         for (QuotaSettings quotaSettings : scanner) {
           final String namespace = quotaSettings.getNamespace();
           final TableName tableName = quotaSettings.getTableName();
@@ -445,13 +453,17 @@ public class SpaceQuotaHelperForTests {
             QuotaUtil.deleteUserQuota(conn, userName);
           }
         }
+      } finally {
+        if (scanner != null) {
+          scanner.close();
+        }
       }
     }
   }
 
   QuotaSettings getTableSpaceQuota(Connection conn, TableName tn) throws IOException {
-    try (QuotaRetriever scanner =
-      new QuotaRetriever(conn, new QuotaFilter().setTableFilter(tn.getNameAsString()))) {
+    try (QuotaRetriever scanner = QuotaRetriever.open(conn.getConfiguration(),
+      new QuotaFilter().setTableFilter(tn.getNameAsString()))) {
       for (QuotaSettings setting : scanner) {
         if (setting.getTableName().equals(tn) && setting.getQuotaType() == QuotaType.SPACE) {
           return setting;
