@@ -53,6 +53,7 @@ import org.apache.hadoop.hbase.testclassification.RegionServerTests;
 import org.apache.hadoop.hbase.util.Bytes;
 import org.apache.hadoop.hbase.util.CommonFSUtils;
 import org.apache.hadoop.hbase.util.Threads;
+import org.apache.hadoop.hbase.wal.FSHLogProvider;
 import org.apache.hadoop.hbase.wal.WAL;
 import org.apache.hadoop.hbase.wal.WALEdit;
 import org.apache.hadoop.hbase.wal.WALKey;
@@ -196,9 +197,10 @@ public class TestFSHLog extends AbstractTestFSWAL {
     }
 
     final String name = this.name.getMethodName();
-    try (CustomFSHLog log = new CustomFSHLog(FS, CommonFSUtils.getRootDir(CONF), name,
-      HConstants.HREGION_OLDLOGDIR_NAME, CONF, null, true, null, null)) {
-      log.setWriter(new FailingWriter());
+    Configuration conf = new Configuration(CONF);
+    conf.set(FSHLogProvider.WRITER_IMPL, FailingWriter.class.getName());
+    try (CustomFSHLog log = new CustomFSHLog(FS, CommonFSUtils.getRootDir(conf), name,
+      HConstants.HREGION_OLDLOGDIR_NAME, conf, null, true, null, null)) {
       Field ringBufferEventHandlerField = FSHLog.class.getDeclaredField("ringBufferEventHandler");
       ringBufferEventHandlerField.setAccessible(true);
       FSHLog.RingBufferEventHandler ringBufferEventHandler =
@@ -208,7 +210,7 @@ public class TestFSHLog extends AbstractTestFSWAL {
       try {
         SyncFuture future0 = log.publishSyncOnRingBuffer();
         // Wait for the sync to be done.
-        Waiter.waitFor(CONF, TEST_TIMEOUT_MS, future0::isDone);
+        Waiter.waitFor(conf, TEST_TIMEOUT_MS, future0::isDone);
         // Publish another sync from the same thread, this should not overwrite the done sync.
         SyncFuture future1 = log.publishSyncOnRingBuffer();
         assertFalse(future1.isDone());
@@ -217,7 +219,7 @@ public class TestFSHLog extends AbstractTestFSWAL {
         // Wait for the safe point to be reached.
         // With the deadlock in HBASE-25984, this is never possible, thus blocking the sync
         // pipeline.
-        Waiter.waitFor(CONF, TEST_TIMEOUT_MS, latch::isSafePointAttained);
+        Waiter.waitFor(conf, TEST_TIMEOUT_MS, latch::isSafePointAttained);
       } finally {
         // Force release the safe point, for the clean up.
         latch.releaseSafePoint();
@@ -317,71 +319,4 @@ public class TestFSHLog extends AbstractTestFSWAL {
       region.close();
     }
   }
-
-  @Test
-  /**
-   * Test for jira https://issues.apache.org/jira/browse/HBASE-28665
-   */
-  public void testWALClosureFailureAndCleanup() throws IOException {
-
-    class FailingWriter implements WALProvider.Writer {
-      @Override
-      public void sync(boolean forceSync) throws IOException {
-
-      }
-
-      @Override
-      public void append(WAL.Entry entry) throws IOException {
-      }
-
-      @Override
-      public long getLength() {
-        return 0;
-      }
-
-      @Override
-      public long getSyncedLength() {
-        return 0;
-      }
-
-      @Override
-      public void close() throws IOException {
-        throw new IOException("WAL close injected failure..");
-      }
-    }
-    final byte[] b = Bytes.toBytes("b");
-    String name = this.name.getMethodName();
-    // Have a FSHLog writer implementation that fails during close
-    try (FSHLog log = new FSHLog(FS, CommonFSUtils.getRootDir(CONF), name,
-      HConstants.HREGION_OLDLOGDIR_NAME, CONF, null, true, null, null)) {
-      log.init();
-
-      // create a region with the wal
-      TableDescriptor htd =
-        TableDescriptorBuilder.newBuilder(TableName.valueOf(this.name.getMethodName()))
-          .setColumnFamily(ColumnFamilyDescriptorBuilder.of(b)).build();
-      RegionInfo hri = RegionInfoBuilder.newBuilder(htd.getTableName()).build();
-      ChunkCreator.initialize(MemStoreLAB.CHUNK_SIZE_DEFAULT, false, 0, 0, 0, null,
-        MemStoreLAB.INDEX_CHUNK_SIZE_PERCENTAGE_DEFAULT);
-      final HRegion region = TEST_UTIL.createLocalHRegion(hri, CONF, htd, log);
-      // Repeat the following steps twice
-      // * Append writes for the FSHLog
-      // * Create a new Writer replace the old writer
-      for (int i = 0; i < 2; i++) {
-        log.setWriter(new FailingWriter());
-        region.put(new Put(b).addColumn(b, b, b));
-        region.put(new Put(b).addColumn(b, b, b));
-        log.rollWriter();
-      }
-
-      region.put(new Put(b).addColumn(b, b, b));
-      region.put(new Put(b).addColumn(b, b, b));
-      region.flush(true);
-      log.rollWriter();
-
-      TEST_UTIL.waitFor(10000, log.walFile2Props::isEmpty);
-      assertEquals("WAL Files not cleaned ", 0, log.walFile2Props.size());
-      region.close();
-    }
-  }
 }
